@@ -1,8 +1,8 @@
-/*						-*- coding: utf-8 -*-
+/*                                              -*- coding: utf-8 -*-
  *
- * c h a r . c				-- Chaacters management
+ * c h a r . c                          -- Characters management
  *
- * Copyright © 1993-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2020 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,12 +23,14 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??????
- * Last file update: 22-Aug-2011 15:58 (eg)
+ * Last file update:  2-Jun-2020 10:47 (eg)
  */
 
 #include <ctype.h>
 #include "stklos.h"
-#include <wctype.h>
+
+
+SCM STk_char_foldcase(SCM c);
 
 
 struct charelem {
@@ -38,10 +40,10 @@ struct charelem {
 
 static struct charelem chartable [] = {
   {"null",       '\000'},
-  {"alarm",	 '\007'}, /* R7RS name */
+  {"alarm",      '\007'}, /* R7RS name */
   {"bell",       '\007'}, /* old STklos name, for backward compatibility */
   {"backspace",  '\010'},
-  {"tab", 	 '\011'},
+  {"tab",        '\011'},
   {"newline",    '\012'},
   {"page",       '\014'},
   {"return",     '\015'},
@@ -86,204 +88,103 @@ static struct charelem chartable [] = {
   {"rs",         '\036'},
   {"us",         '\037'},
 
-  {"sp",	'\040'},
-  {"del",	'\177'},
+  {"sp",        '\040'},
+  {"del",       '\177'},
 
   {"",           '\000'}
 };
 
-
-static struct {
-  uint16_t ch;
-  uint16_t fold;
-} casefold_table [] = {
-  { 0xb5, 0x3bc },       /*  µ -> μ */
-  { 0x17f, 0x73 },       /* ſ -> s */
-  { 0x345, 0x3b9 },      /*  ͅ -> ι */
-  { 0x3c2, 0x3c3 },      /* ς -> σ */
-  { 0x3cf, 0x3d7 },      /*  Ϗ -> ϗ */
-  { 0x3d0, 0x3b2 },      /* ϐ -> β */
-  { 0x3d1, 0x3b8 },      /*  ϑ -> θ */
-  { 0x3d5, 0x3c6 },      /* ϕ -> φ */
-  { 0x3d6, 0x3c0 },      /*  ϖ -> π */
-  { 0x3f0, 0x3ba },      /* ϰ -> κ */
-  { 0x3f1, 0x3c1 },      /* ϱ -> ρ */
-  { 0x3f5, 0x3b5 },      /* ϵ -> ε */
-  { 0x524, 0x525 },      /* Ԥ -> ԥ */
-  { 0x526, 0x527 },      /* Ԧ -> ԧ */
-  { 0x1e9b, 0x1e61 },    /* ẛ -> ṡ */
-  { 0x1fbe, 0x3b9 },     /* ι -> ι */
-  { 0x2c70, 0x252 },     /* Ɒ -> ɒ */
-  { 0x2c7e, 0x23f },     /* Ȿ -> ȿ */
-  { 0x2c7f, 0x240 },     /* Ɀ -> ɀ */
-  { 0x2ceb, 0x2cec },    /* Ⳬ -> ⳬ */
-  { 0x2ced, 0x2cee },    /* Ⳮ -> ⳮ */
-  { 0xa640, 0xa641 },    /* Ꙁ -> ꙁ */
-  { 0xa642, 0xa643 },    /* Ꙃ -> ꙃ */
-  { 0xa644, 0xa645 },    /* Ꙅ -> ꙅ */
-  { 0xa646, 0xa647 },    /* Ꙇ -> ꙇ */
-  { 0xa648, 0xa649 },    /* Ꙉ -> ꙉ */
-  { 0xa64a, 0xa64b },    /* Ꙋ -> ꙋ */
-  { 0xa64c, 0xa64d },    /* Ꙍ -> ꙍ */
-  { 0xa64e, 0xa64f },    /* Ꙏ -> ꙏ */
-  { 0xa650, 0xa651 },    /* Ꙑ -> ꙑ */
-  { 0xa652, 0xa653 },    /* Ꙓ -> ꙓ */
-  { 0xa654, 0xa655 },    /* Ꙕ -> ꙕ */
-  { 0xa656, 0xa657 },    /* Ꙗ -> ꙗ */
-  { 0xa658, 0xa659 },    /* Ꙙ -> ꙙ */
-  { 0xa65a, 0xa65b },    /* Ꙛ -> ꙛ */
-  { 0xa65c, 0xa65d },    /* Ꙝ -> ꙝ */
-  { 0xa65e, 0xa65f },    /* Ꙟ -> ꙟ */
-  { 0xa660, 0xa661 },    /* Ꙡ -> ꙡ */
-  { 0xa662, 0xa663 },    /* Ꙣ -> ꙣ */
-  { 0xa664, 0xa665 },    /* Ꙥ -> ꙥ */
-  { 0xa666, 0xa667 },    /* Ꙧ -> ꙧ */
-  { 0xa668, 0xa669 },    /* Ꙩ -> ꙩ */
-  { 0xa66a, 0xa66b },    /* Ꙫ -> ꙫ */
-  { 0xa66c, 0xa66d },    /* Ꙭ -> ꙭ */
-  { 0xa680, 0xa681 },    /* Ꚁ -> ꚁ */
-  { 0xa682, 0xa683 },    /* Ꚃ -> ꚃ */
-  { 0xa684, 0xa685 },    /* Ꚅ -> ꚅ */
-  { 0xa686, 0xa687 },    /* Ꚇ -> ꚇ */
-  { 0xa688, 0xa689 },    /* Ꚉ -> ꚉ */
-  { 0xa68a, 0xa68b },    /* Ꚋ -> ꚋ */
-  { 0xa68c, 0xa68d },    /* Ꚍ -> ꚍ */
-  { 0xa68e, 0xa68f },    /* Ꚏ -> ꚏ */
-  { 0xa690, 0xa691 },    /* Ꚑ -> ꚑ */
-  { 0xa692, 0xa693 },    /* Ꚓ -> ꚓ */
-  { 0xa694, 0xa695 },    /* Ꚕ -> ꚕ */
-  { 0xa696, 0xa697 },    /* Ꚗ -> ꚗ */
-  { 0xa722, 0xa723 },    /* Ꜣ -> ꜣ */
-  { 0xa724, 0xa725 },    /* Ꜥ -> ꜥ */
-  { 0xa726, 0xa727 },    /* Ꜧ -> ꜧ */
-  { 0xa728, 0xa729 },    /* Ꜩ -> ꜩ */
-  { 0xa72a, 0xa72b },    /* Ꜫ -> ꜫ */
-  { 0xa72c, 0xa72d },    /* Ꜭ -> ꜭ */
-  { 0xa72e, 0xa72f },    /* Ꜯ -> ꜯ */
-  { 0xa732, 0xa733 },    /* Ꜳ -> ꜳ */
-  { 0xa734, 0xa735 },    /* Ꜵ -> ꜵ */
-  { 0xa736, 0xa737 },    /* Ꜷ -> ꜷ */
-  { 0xa738, 0xa739 },    /* Ꜹ -> ꜹ */
-  { 0xa73a, 0xa73b },    /* Ꜻ -> ꜻ */
-  { 0xa73c, 0xa73d },    /* Ꜽ -> ꜽ */
-  { 0xa73e, 0xa73f },    /* Ꜿ -> ꜿ */
-  { 0xa740, 0xa741 },    /* Ꝁ -> ꝁ */
-  { 0xa742, 0xa743 },    /* Ꝃ -> ꝃ */
-  { 0xa744, 0xa745 },    /* Ꝅ -> ꝅ */
-  { 0xa746, 0xa747 },    /* Ꝇ -> ꝇ */
-  { 0xa748, 0xa749 },    /* Ꝉ -> ꝉ */
-  { 0xa74a, 0xa74b },    /* Ꝋ -> ꝋ */
-  { 0xa74c, 0xa74d },    /* Ꝍ -> ꝍ */
-  { 0xa74e, 0xa74f },    /* Ꝏ -> ꝏ */
-  { 0xa750, 0xa751 },    /* Ꝑ -> ꝑ */
-  { 0xa752, 0xa753 },    /* Ꝓ -> ꝓ */
-  { 0xa754, 0xa755 },    /* Ꝕ -> ꝕ */
-  { 0xa756, 0xa757 },    /* Ꝗ -> ꝗ */
-  { 0xa758, 0xa759 },    /* Ꝙ -> ꝙ */
-  { 0xa75a, 0xa75b },    /* Ꝛ -> ꝛ */
-  { 0xa75c, 0xa75d },    /* Ꝝ -> ꝝ */
-  { 0xa75e, 0xa75f },    /* Ꝟ -> ꝟ */
-  { 0xa760, 0xa761 },    /* Ꝡ -> ꝡ */
-  { 0xa762, 0xa763 },    /* Ꝣ -> ꝣ */
-  { 0xa764, 0xa765 },    /* Ꝥ -> ꝥ */
-  { 0xa766, 0xa767 },    /* Ꝧ -> ꝧ */
-  { 0xa768, 0xa769 },    /* Ꝩ -> ꝩ */
-  { 0xa76a, 0xa76b },    /* Ꝫ -> ꝫ */
-  { 0xa76c, 0xa76d },    /* Ꝭ -> ꝭ */
-  { 0xa76e, 0xa76f },    /* Ꝯ -> ꝯ */
-  { 0xa779, 0xa77a },    /* Ꝺ -> ꝺ */
-  { 0xa77b, 0xa77c },    /* Ꝼ -> ꝼ */
-  { 0xa77d, 0x1d79 },    /* Ᵹ -> ᵹ */
-  { 0xa77e, 0xa77f },    /* Ꝿ -> ꝿ */
-  { 0xa780, 0xa781 },    /* Ꞁ -> ꞁ */
-  { 0xa782, 0xa783 },    /* Ꞃ -> ꞃ */
-  { 0xa784, 0xa785 },    /* Ꞅ -> ꞅ */
-  { 0xa786, 0xa787 },    /* Ꞇ -> ꞇ */
-  { 0xa78b, 0xa78c },    /* Ꞌ -> ꞌ */
-  { 0xa78d, 0x265 },     /* Ɥ -> ɥ */
-  { 0xa790, 0xa791 },    /* Ꞑ -> ꞑ */
-  { 0xa7a0, 0xa7a1 },    /* Ꞡ -> ꞡ */
-  { 0xa7a2, 0xa7a3 },    /* Ꞣ -> ꞣ */
-  { 0xa7a4, 0xa7a5 },    /* Ꞥ -> ꞥ */
-  { 0xa7a6, 0xa7a7 },    /* Ꞧ -> ꞧ */
-  { 0xa7a8, 0xa7a9 }     /* Ꞩ -> ꞩ */
+struct utf8_conversion_char {
+  uint32_t key;
+  uint32_t val;
 };
 
+typedef uint32_t utf8_char;
 
-int STk_casefold_char(int ch)
-{
-  static int min = -1, max= -1, len = -1;
+#include "utf8-tables.inc"
 
-  if (len == -1)  {
-    /* Never run before. Initialize static variables */
-    len = sizeof(casefold_table) / sizeof(casefold_table[0]);
-    min = casefold_table[0].ch;
-    max = casefold_table[len-1].ch;
-  }
+static int search_conversion_table(unsigned int ch, struct utf8_conversion_char table[],
+                                   int len) {
+  unsigned int min = table[0].key;
+  unsigned int max = table[len-1].key;
 
   if (min <= ch && ch <= max) {
-    /* seach the value in the casefold_table by dichotomy */
+    /* seach the value in the table by dichotomy */
     int left, right, i;
 
     left = 0; right = len-1;
     do {
       i = (left + right) / 2;
-      if (ch == casefold_table[i].ch)
-	return casefold_table[i].fold;
+      if (ch == table[i].key)
+        return table[i].val;
       else
-	if (ch < casefold_table[i].ch)
-	  right = i -1;
-	else
-	  left = i +1;
+        if (ch < table[i].key)
+          right = i-1;
+        else
+          left = i+1;
     }
     while (left <= right);
   }
+  /* not found of not in the interval of special character => return -1 */
+  return -1;
+}
 
-  /* not found of not in the interval of special character => return the
-   * corresponding lowercase character
-   */
-  return towlower(ch);
+static int search_ordered_list(utf8_char ch,  utf8_char table[], int len) {
+  unsigned int min = table[0];
+  unsigned int max = table[len-1];
+
+  if (min <= ch && ch <= max) {
+    /* seach the value in the table by dichotomy */
+    int left, right, i;
+
+    left = 0; right = len-1;
+    do {
+      i = (left + right) / 2;
+      if (ch == table[i])
+        return table[i];
+      else
+        if (ch < table[i])
+          right = i-1;
+        else
+          left = i+1;
+    }
+    while (left <= right);
+  }
+  /* not found of not in the interval of special character => return -1 */
+  return -1;
 }
 
 
 /*===========================================================================*\
  *
- * 				     Utilities
+ *                                   Utilities
  *
 \*===========================================================================*/
 
-static int my_strcmpi(register char *p1, register char *p2)
-{
-  for( ; tolower(*p1) == tolower(*p2); p1++, p2++)
-    if (!*p1) return 0;
-  return tolower(*p1) - tolower(*p2);
-}
-
-
 static void error_bad_char(SCM c)
 {
   STk_error("bad char", c);
 }
 
-static int charcomp(SCM c1, SCM c2)
+static Inline int charcomp(SCM c1, SCM c2)
 {
-  if (!CHARACTERP(c1)) error_bad_char(c1);
-  if (!CHARACTERP(c2)) error_bad_char(c2);
   return (CHARACTER_VAL(c1) - CHARACTER_VAL(c2));
 }
 
 
 static int charcompi(SCM c1, SCM c2)
 {
-  if (!CHARACTERP(c1)) error_bad_char(c1);
-  if (!CHARACTERP(c2)) error_bad_char(c2);
   return STk_use_utf8 ?
-         (STk_casefold_char(CHARACTER_VAL(c1)) -
-	  STk_casefold_char(CHARACTER_VAL(c2))):
-         (tolower((unsigned char) CHARACTER_VAL(c1)) -
-	  tolower((unsigned char) CHARACTER_VAL(c2)));
+    (int) (STk_to_fold(CHARACTER_VAL(c1)) -
+           STk_to_fold(CHARACTER_VAL(c2))):
+    (int) (tolower((unsigned char) CHARACTER_VAL(c1)) -
+           tolower((unsigned char) CHARACTER_VAL(c2)));
 }
 
+/* Comparison of characters. No test on types */
+int STk_charcomp(SCM c1, SCM c2)  { return charcomp(c1,  c2);  }
+int STk_charcompi(SCM c1, SCM c2) { return charcompi(c1,  c2); }
+
 
 int STk_string2char(char *s)
 /* converts a char name to a char */
@@ -304,15 +205,15 @@ int STk_string2char(char *s)
   }
 
   for (p=chartable; *(p->name); p++) {
-    if (my_strcmpi(p->name, s) == 0) return (int) (p->value);
+    if (strcasecmp(p->name, s) == 0) return (int) (p->value);
   }
   STk_error("bad char name %S", s);
   return 0; /* never reached */
 }
 
 
-char *STk_char2string(int c)  		/* convert a character to it's */
-{					/* external representation */
+char *STk_char2string(int c)            /* convert a character to it's */
+{                                       /* external representation */
   register struct charelem *p;
 
   for (p=chartable; *(p->name); p++)
@@ -325,7 +226,7 @@ char *STk_char2string(int c)  		/* convert a character to it's */
 
 /*===========================================================================*\
  *
- * 				     PRIMITIVES
+ *                                   PRIMITIVES
  *
 \*===========================================================================*/
 
@@ -343,20 +244,13 @@ doc>
 
 /*=============================================================================*/
 
-
-#define CHAR_COMPARE(sname, cname, tst)				\
-   DEFINE_PRIMITIVE(sname, cname, subr2, (SCM c1, SCM c2))	\
-   {								\
-     return MAKE_BOOLEAN(tst);					\
-   }
-
 /*
 <doc char=? char<? char>? char<=? char>=?
- * (char=? char1 char2)
- * (char<? char1 char2)
- * (char>? char1 char2)
- * (char<=? char1 char2)
- * (char>=? char1 char2)
+ * (char=? char1 char2 ...)
+ * (char<? char1 char2 ...)
+ * (char>? char1 char2 ...)
+ * (char<=? char1 char2 ...)
+ * (char>=? char1 char2 ...)
  *
  * These procedures impose a total ordering on the set of characters.
  * It is guaranteed that under this ordering:
@@ -370,45 +264,46 @@ doc>
 doc>
  */
 
-CHAR_COMPARE("char=?",  chareq, (charcomp(c1,c2) == 0))
-CHAR_COMPARE("char<?",  charlt, (charcomp(c1,c2) <  0))
-CHAR_COMPARE("char>?",  chargt, (charcomp(c1,c2) >  0))
-CHAR_COMPARE("char<=?", charle, (charcomp(c1,c2) <= 0))
-CHAR_COMPARE("char>=?", charge, (charcomp(c1,c2) >= 0))
+#define CHAR_COMPARE(sname, name, tst)                         \
+  DEFINE_PRIMITIVE(sname, name, vsubr, (int argc, SCM *argv)) { \
+    SCM last;                                                   \
+                                                                \
+    if (!argc) STk_error("need at least one parameter");        \
+    if (!CHARACTERP(*argv)) error_bad_char(*argv);              \
+                                                                \
+    for (last = *argv--; --argc; last=*argv--) {                \
+      if (!CHARACTERP(*argv)) error_bad_char(*argv);            \
+      if (tst) return STk_false;                                \
+    }                                                           \
+    return STk_true;                                            \
+  }
+
+CHAR_COMPARE("char=?",  chareq, (charcomp(last, *argv) != 0))
+CHAR_COMPARE("char<?",  charlt, (charcomp(last, *argv) >= 0))
+CHAR_COMPARE("char>?",  chargt, (charcomp(last, *argv) <= 0))
+CHAR_COMPARE("char<=?", charle, (charcomp(last, *argv) > 0))
+CHAR_COMPARE("char>=?", charge, (charcomp(last, *argv) < 0))
 
 
 /*
 <doc char-ci=? char-ci<? char-ci>? char-ci<=? char-ci>=?
- * (char-ci=? char1 char2)
- * (char-ci<? char1 char2)
- * (char-ci>? char1 char2)
- * (char-ci<=? char1 char2)
- * (char-ci>=? char1 char2)
+ * (char-ci=? char1 char2 ...)
+ * (char-ci<? char1 char2 ...)
+ * (char-ci>? char1 char2 ...)
+ * (char-ci<=? char1 char2 ...)
+ * (char-ci>=? char1 char2 ...)
  *
  * These procedures are similar to |char=?| et cetera, but they treat
  * upper case and lower case letters as the same. For example,
  * |(char-ci=? #\A #\a)| returns |#t|.
 doc>
  */
+CHAR_COMPARE("char-ci=?",  chareqi, (charcompi(last,*argv) != 0))
+CHAR_COMPARE("char-ci<?",  charlti, (charcompi(last,*argv) >= 0))
+CHAR_COMPARE("char-ci>?",  chargti, (charcompi(last,*argv) <= 0))
+CHAR_COMPARE("char-ci<=?", charlei, (charcompi(last,*argv) > 0))
+CHAR_COMPARE("char-ci>=?", chargei, (charcompi(last,*argv) < 0))
 
-CHAR_COMPARE("char-ci=?",  chareqi, (charcompi(c1,c2) == 0))
-CHAR_COMPARE("char-ci<?",  charlti, (charcompi(c1,c2) <  0))
-CHAR_COMPARE("char-ci>?",  chargti, (charcompi(c1,c2) >  0))
-CHAR_COMPARE("char-ci<=?", charlei, (charcompi(c1,c2) <= 0))
-CHAR_COMPARE("char-ci>=?", chargei, (charcompi(c1,c2) >= 0))
-
-
-/*=============================================================================*/
-
-
-#define TEST_CTYPE(tst, name)                                                       \
-   DEFINE_PRIMITIVE(name, CPP_CONCAT(char_is, tst), subr1, (SCM c))                 \
-   {                                                                                \
-     if (!CHARACTERP(c)) error_bad_char(c);                                         \
-     return STk_use_utf8 ?                                                          \
-              MAKE_BOOLEAN(CPP_CONCAT(isw, tst)(CHARACTER_VAL(c))):                 \
-              MAKE_BOOLEAN(CPP_CONCAT(is,  tst)((unsigned char) CHARACTER_VAL(c))); \
-   }
 
 /*
 <doc char-alphabetic? char-numeric? char-whitespace? char-upper-case? char-lower-case?
@@ -428,13 +323,90 @@ CHAR_COMPARE("char-ci>=?", chargei, (charcompi(c1,c2) >= 0))
 doc>
  */
 
-TEST_CTYPE(alpha, "char-alphabetic?")
-TEST_CTYPE(digit, "char-numeric?")
-TEST_CTYPE(space, "char-whitespace?")
-TEST_CTYPE(upper, "char-upper-case?")
-TEST_CTYPE(lower, "char-lower-case?")
+DEFINE_PRIMITIVE("char-alphabetic?", char_isalpha, subr1, (SCM c)) {
+  if (!CHARACTERP(c)) error_bad_char(c);
+  if (STk_use_utf8)
+    return MAKE_BOOLEAN(-1 != search_ordered_list(CHARACTER_VAL(c),
+                                                  letters_table,
+                                                  letters_table_length));
+  else
+    return MAKE_BOOLEAN(isalpha(CHARACTER_VAL(c)));
+}
 
 
+DEFINE_PRIMITIVE("char-numeric?", char_isdigit, subr1, (SCM c)) {
+  if (!CHARACTERP(c)) error_bad_char(c);
+  if (STk_use_utf8)
+    return MAKE_BOOLEAN(-1 != search_conversion_table(CHARACTER_VAL(c),
+                                                      digits_table,
+                                                      digits_table_length));
+  else
+    return MAKE_BOOLEAN(isdigit(CHARACTER_VAL(c)));
+}
+
+
+DEFINE_PRIMITIVE("char-whitespace?", char_isspace, subr1, (SCM c)) {
+  if (!CHARACTERP(c)) error_bad_char(c);
+  if (STk_use_utf8)
+    return MAKE_BOOLEAN(-1 != search_ordered_list(CHARACTER_VAL(c),
+                                                  spaces_table,
+                                                  spaces_table_length));
+  else
+    return MAKE_BOOLEAN(isspace(CHARACTER_VAL(c)));
+}
+
+DEFINE_PRIMITIVE("char-upper-case?", char_isupper, subr1, (SCM c)) {
+  if (!CHARACTERP(c)) error_bad_char(c);
+  if (STk_use_utf8)
+    return MAKE_BOOLEAN(-1 != search_conversion_table(CHARACTER_VAL(c),
+                                                      upper_table,
+                                                      upper_table_length));
+  else
+    return MAKE_BOOLEAN(isupper(CHARACTER_VAL(c)));
+}
+
+
+DEFINE_PRIMITIVE("char-lower-case?", char_islower, subr1, (SCM c)) {
+  if (!CHARACTERP(c)) error_bad_char(c);
+  if (STk_use_utf8)
+    return MAKE_BOOLEAN(-1 != search_conversion_table(CHARACTER_VAL(c),
+                                                      lower_table,
+                                                      lower_table_length));
+  else
+    return MAKE_BOOLEAN(islower(CHARACTER_VAL(c)));
+}
+
+/*
+<doc R7RS digit-value
+ * (digit-value char)
+ *
+ * This procedure returns the numeric value (0 to 9) of its
+ * argument if it is a numeric digit (that is, if char-numeric?
+ * returns #t), or #f on any other character.
+ * @lisp
+ * (digit-value
+ * (digit-value #\3)        => 3
+ * (digit-value #\x0664)    => 4
+ * (digit-value #\x0AE6)    => 0
+ * (digit-value #\x0EA6)    #f
+ * @end lisp
+doc>
+ */
+
+DEFINE_PRIMITIVE("digit-value", digit_value, subr1, (SCM c))
+{
+  int val;
+
+  if (!CHARACTERP(c)) error_bad_char(c);
+
+  val = CHARACTER_VAL(c);
+  if (STk_use_utf8) {
+    int res = search_conversion_table(val, digits_table, digits_table_length);
+    return (res == -1) ? STk_false: MAKE_INT(res);
+  } else {
+    return (('0' <= val) && (val <= '9')) ? MAKE_INT(val - '0'): STk_false;
+  }
+}
 /*=============================================================================*/
 
 /*
@@ -480,7 +452,7 @@ DEFINE_PRIMITIVE("integer->char", integer2char, subr1, (SCM i))
     /* Unicode defines characters in the range [0, #xd7FF] U [#xE000, #x10FFFF] */
     if (! VALID_UTF8_VALUE(c))
       STk_error("bad integer ~S (must be in range [0, #xd7FF] U [#xE000, #x10FFFF]",
-		i);
+                i);
   }
   else
     /* Monobyte character: use them in the range [0, #xFF] */
@@ -503,20 +475,33 @@ DEFINE_PRIMITIVE("integer->char", integer2char, subr1, (SCM i))
  * lower case.
 doc>
  */
+uint32_t STk_to_upper(uint32_t c) {
+  if (STk_use_utf8) {
+    int res = search_conversion_table(c, lower_table, lower_table_length);
+    return (res <= 0) ? c : (uint32_t) res;   // -1: not a lowercase, 0 lower without upper
+  } else
+    return toupper(c);
+}
+
+uint32_t STk_to_lower(uint32_t c) {
+  if (STk_use_utf8) {
+    int res = search_conversion_table(c, upper_table, upper_table_length);
+    return (res <=0) ? c : (uint32_t) res;    // -1: not a lowercase, 0 lower without lower
+  } else
+    return tolower(c);
+}
+
 DEFINE_PRIMITIVE("char-upcase", char_upcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c)) error_bad_char(c);
-  return MAKE_CHARACTER(STk_use_utf8 ?
-			towupper(CHARACTER_VAL(c)):
-			toupper((unsigned char) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_upper((uint32_t) CHARACTER_VAL(c)));
 }
 
+
 DEFINE_PRIMITIVE("char-downcase", char_downcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c)) error_bad_char(c);
-  return MAKE_CHARACTER(STk_use_utf8 ?
-			towlower(CHARACTER_VAL(c)) :
-			tolower((unsigned char) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_lower((uint32_t) CHARACTER_VAL(c)));
 }
 /*
 <doc EXT char-foldcase
@@ -529,12 +514,18 @@ DEFINE_PRIMITIVE("char-downcase", char_downcase, subr1, (SCM c))
  * does not exist.
 doc>
  */
+uint32_t STk_to_fold(uint32_t c) {
+  if (STk_use_utf8) {
+    int res = search_conversion_table(c, fold_table, fold_table_length);
+    return (res <=0) ? STk_to_lower(c) : (uint32_t) res; 
+  } else
+    return tolower(c);
+}
+
 DEFINE_PRIMITIVE("char-foldcase", char_foldcase, subr1, (SCM c))
 {
   if (!CHARACTERP(c))  error_bad_char(c);
-  return MAKE_CHARACTER(STk_use_utf8 ?
-			STk_casefold_char(CHARACTER_VAL(c)):
-			tolower((unsigned char) CHARACTER_VAL(c)));
+  return MAKE_CHARACTER(STk_to_fold((uint32_t) CHARACTER_VAL(c)));
 }
 
 
@@ -560,6 +551,8 @@ int STk_init_char(void)
   ADD_PRIMITIVE(char_isupper);
   ADD_PRIMITIVE(char_islower);
 
+  ADD_PRIMITIVE(digit_value);
+
 
   ADD_PRIMITIVE(char2integer);
   ADD_PRIMITIVE(integer2char);
