@@ -1,8 +1,8 @@
 /*
  *
- * e r r o r . c 			-- The error procedure
+ * e r r o r . c                        -- The error procedure
  *
- * Copyright © 1993-2007 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,17 +22,16 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 14-Nov-1993 14:58
- * Last file update: 30-May-2007 16:27 (eg)
+ * Last file update: 26-Mar-2021 11:01 (eg)
  */
 
 #include "stklos.h"
-
+#include "struct.h"
 
 /*===========================================================================*\
  *
  * A simplified (and very ad hoc) version of printf for error messages.
- * Recognized formats:
- *       %c for character
+  *       %c for character
  *       %s for string
  *       %S for string (surrounded by a pair of quotes)
  *       %d for decimal print
@@ -40,14 +39,14 @@
  *       %% for printing a '%'
  *       ~A for printing a Scheme object in display mode
  *       ~S for printing a Scheme object in write mode
- *	 ~W for printing a Scheme object in write mode (circular)
- *	 ~% for printing a newline
+ *       ~W for printing a Scheme object in write mode (circular)
+ *       ~% for printing a newline
  *       ~~ for printing a tilde character
  *
 \*===========================================================================*/
 
 
-static void print_int(SCM port, unsigned int x, int base)
+static void print_int(SCM port, unsigned int x, unsigned int base)
 {
   if (x >= base) print_int(port, x / base, base);
   x %= base;
@@ -64,50 +63,50 @@ static void print_format(SCM port,char *format, va_list ap)
     if (*s == '%') {
       /* % format (C-like) */
       switch (*++s) {
-	case '%': STk_putc('%', port); break;
+        case '%': STk_putc('%', port); break;
 
-	case 'S': STk_putc('`', port); /* No break */
-	case 's': for (str = va_arg(ap, char *); *str; str++)
-		    STk_putc(*str, port);
-		  if (*s == 'S') STk_putc('\'', port);
-		  break;
-	case 'c': STk_putc(va_arg(ap, int), port); break;
-	case 'x': print_int(port, va_arg(ap, unsigned int), 16); break;
-	case 'd': {
-	  	    int val =  va_arg(ap, unsigned int);
+        case 'S': STk_putc('`', port); /* FALLTHROUGH */
+        case 's': for (str = va_arg(ap, char *); *str; str++)
+                    STk_putc(*str, port);
+                  if (*s == 'S') STk_putc('\'', port);
+                  break;
+        case 'c': STk_putc(va_arg(ap, int), port); break;
+        case 'x': print_int(port, va_arg(ap, unsigned int), 16); break;
+        case 'd': {
+                    int val =  va_arg(ap, unsigned int);
 
-		    if (val < 0) {
-		      STk_putc('-', port);
-		      print_int(port, -val, 10);
-		    }
-		    else
-		      print_int(port, val, 10);
-		    break;
-		  }
-	default:  STk_putc('%', port);
-		  if (*s) STk_putc(*s, port);
-		  break;
+                    if (val < 0) {
+                      STk_putc('-', port);
+                      print_int(port, -val, 10);
+                    }
+                    else
+                      print_int(port, val, 10);
+                    break;
+                  }
+        default:  STk_putc('%', port);
+                  if (*s) STk_putc(*s, port);
+                  break;
       }
     } else if (*s == '~') {
       /* ~ format (CL like) */
       switch (*++s) {
-	case 'A': STk_putc('`', port); /* No break */
-	case 'a': STk_print(va_arg(ap, SCM), port, DSP_MODE);
-		  if (*s == 'A') STk_putc('\'', port);
-		  break;
-	case 'W': STk_putc('`', port);	/* No break */
-	case 'w': STk_print_star(va_arg(ap, SCM), port);
-	  	  if (*s == 'W') STk_putc('\'', port);
-		  break;
-	case 'S': STk_putc('`', port);	/* No break */
-	case 's': STk_print(va_arg(ap, SCM), port, WRT_MODE);
-		  if (*s == 'S') STk_putc('\'', port);
-		  break;
-	case '~': STk_putc('~', port);  break;
-	case '%': STk_putc('\n', port); break;
-	default:  STk_putc('~', port);
-	  	  if (*s) STk_putc(*s, port);
-		  break;
+        case 'A': STk_putc('`', port); /* FALLTHROUGH */
+        case 'a': STk_print(va_arg(ap, SCM), port, DSP_MODE);
+                  if (*s == 'A') STk_putc('\'', port);
+                  break;
+        case 'W': STk_putc('`', port);  /* FALLTHROUGH */
+        case 'w': STk_print_star(va_arg(ap, SCM), port,WRT_MODE);
+                  if (*s == 'W') STk_putc('\'', port);
+                  break;
+        case 'S': STk_putc('`', port);  /* FALLTHROUGH */
+        case 's': STk_print(va_arg(ap, SCM), port, WRT_MODE);
+                  if (*s == 'S') STk_putc('\'', port);
+                  break;
+        case '~': STk_putc('~', port);  break;
+        case '%': STk_putc('\n', port); break;
+        default:  STk_putc('~', port);
+                  if (*s) STk_putc(*s, port);
+                  break;
       }
     } else {
       /* Normal character */
@@ -116,11 +115,12 @@ static void print_format(SCM port,char *format, va_list ap)
   }
 }
 
-void STk_signal_error(SCM type, SCM where, SCM str)
+void STk_signal_error(SCM type, SCM where, SCM str, SCM msg, SCM irritants)
 {
   SCM bt = STk_vm_bt();
 
-  STk_raise_exception(STk_make_C_cond(type, 3, where, bt, str));
+  STk_raise_exception(STk_make_C_cond(type, 5, where, bt, str,
+                                      msg, irritants));
 }
 
 SCM STk_format_error(char *format, ...)
@@ -145,7 +145,7 @@ static SCM make_error_condition(char *format, va_list ap)
 {
   SCM out, bt;
 
-  /* Grab a baktrace */
+  /* Grab a backtrace */
   bt = STk_vm_bt();
 
   /* Open a string port */
@@ -155,12 +155,14 @@ static SCM make_error_condition(char *format, va_list ap)
   print_format(out, format, ap);
 
   /* and return error */
+  out =  STk_get_output_string(out);
   return STk_make_C_cond(STk_err_mess_condition,
-                         3,
+                         5,
                          STk_false, /* no location */
                          bt,
-                         STk_get_output_string(out));
-
+                         out,       /* formatted output */
+                         out,       /* here too (because a C error */
+                         STk_nil);  /* may contain C objects */
 }
 
 
@@ -178,8 +180,6 @@ SCM STk_make_error(char *format, ...)
 }
 
 
-
-
 void STk_error(char *format, ...)
 {
   va_list ap;
@@ -194,6 +194,21 @@ void STk_error(char *format, ...)
 }
 
 
+void STk_error_with_location(SCM loc, char *format, ...)
+{
+  va_list ap;
+  SCM cond;
+
+  va_start(ap, format);
+  cond = make_error_condition(format, ap);
+  STk_int_struct_set(cond, STk_intern("location"), loc);
+  va_end(ap);
+
+  /* Signal error */
+  STk_raise_exception(cond);
+}
+
+
 void STk_warning(char *format, ...)
 {
   va_list ap;
@@ -237,8 +252,8 @@ void STk_panic(char *format, ...)
 void STk_signal(char *str)
 {
   STk_raise_exception(STk_make_C_cond(STk_message_condition,
-				      1,
-				      STk_Cstring2string(str)));
+                                      1,
+                                      STk_Cstring2string(str)));
 }
 
 
@@ -260,7 +275,7 @@ void STk_debug(char *format, ...)
   STk_flush(eport);
 }
 
-void STk_gdb(SCM obj)		/* associated to the  gdb write function */
+void STk_gdb(SCM obj)           /* associated to the  gdb write function */
 {
   STk_debug("Object 0x%lx value = ~s", (unsigned long) obj, obj);
 }
