@@ -1,7 +1,7 @@
 /*
- * f p o r t . c				-- File ports
+ * f p o r t . c                                -- File ports
  *
- * Copyright © 2000-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2000-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,14 +14,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should hcave received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  * USA.
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  8-Jan-2000 14:48 (eg)
- * Last file update: 27-May-2011 22:33 (eg)
+ * Last file update: 28-Aug-2021 11:45 (eg)
  *
  * This implementation is built by reverse engineering on an old SUNOS 4.1.1
  * stdio.h. It has been simplified to fit the needs for STklos. In particular
@@ -30,13 +30,69 @@
  *
  */
 #include <ctype.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/wait.h>
+
 #include "stklos.h"
 #include "fport.h"
 #include "vm.h"
 
-int STk_interactive = 0;		  /* We are in interactive mode */
-SCM STk_stdin, STk_stdout, STk_stderr;	  /* The unredirected ports */
-MUT_DECL(all_fports_mutex);
+int STk_interactive = 0;                  /* We are in interactive mode */
+SCM STk_stdin, STk_stdout, STk_stderr;    /* The unredirected ports */
+MUT_DECL(all_fports_mutex)
+
+/*
+ * Implementation of our own popen/pclose. We use her file descriptor, instad
+ * of FILE *.
+ *
+ * The fd_popen function returns also the pid of the shell process launched
+ * for the redirection. This pid is stored in the internal file port
+ * representation to wait on it during the fd_pclose.
+ */
+static int fd_popen(char *cmd, char *mode, int *pid)
+{
+  int p[2];
+
+  if ((*mode != 'r' && *mode != 'w') || mode[1] ||
+      pipe(p) == -1) {
+    return -1;
+  }
+
+  switch (*pid = fork()) {
+  case -1:                                   /* Cannot fork */
+      close(p[0]); close(p[1]);
+      return -1;
+    case 0:                                  /* Child process */
+      if (*mode == 'r') {
+        if (p[1] != 1) dup2(p[1], 1);
+        close(p[0]);
+      } else {
+        if (p[0] != 0) dup2(p[0], 0);
+        close(p[1]);
+      }
+      execlp("sh", "sh", "-c", cmd, NULL);
+      STk_panic("*** shell error in fd_popen");
+      return -1; /* for the compiler */
+    default:                                 /* Parent process*/
+      if (*mode == 'r') {
+        close(p[1]);
+        return p[0];
+      } else {
+        close(p[0]);
+        return p[1];
+      }
+    }
+}
+
+static int fd_pclose(int fd, int pid) {
+  int status, n;
+  close(fd);
+  do {
+    n = waitpid(pid, &status, 0);
+  } while (n == -1 && errno == EINTR);
+  return WEXITSTATUS(status);
+}
 
 
 /*
@@ -56,7 +112,7 @@ static struct port_list {
 } *all_file_ports = NULL;
 
 
-static void nop_release_port(SCM port)
+static void nop_release_port(SCM _UNUSED(port))
 {
   /* This function is used as a release procedure when closing all files
    * at program exit. We use an empty procedure to avoid interferences
@@ -84,9 +140,9 @@ static void unregister_port(SCM port)
   for (prev = cur = all_file_ports; cur ; prev=cur, cur=cur->next) {
     if (GET_FAKE_POINTER(cur->port) == port) {
       if (cur == prev)
-	all_file_ports = all_file_ports->next;
+        all_file_ports = all_file_ports->next;
       else
-	prev->next = cur->next;
+        prev->next = cur->next;
       break;
     }
   }
@@ -105,7 +161,7 @@ void STk_close_all_ports(void)
     PORT_RELEASE(tmp) = nop_release_port;
     if (OPORTP(tmp)) {
       if (tmp != eport && tmp != oport)
-	STk_close(tmp);
+        STk_close(tmp);
     }
   }
   MUT_UNLOCK(all_fports_mutex);
@@ -117,13 +173,13 @@ void STk_close_all_ports(void)
 
 /*===========================================================================*\
  *
- * 					Utils
+ *                                      Utils
  *
 \*===========================================================================*/
 
 static void fill_buffer(struct fstream *f)
 {
-  int n = 0; 				/* to avoid gcc warning */
+  int n = 0;                            /* to avoid gcc warning */
   unsigned char *ptr = PORT_BASE(f);
 
   /* Interactive ports can see multiple EOF, so clear EOF flag  */
@@ -203,7 +259,7 @@ static Inline int Fgetc(void *stream)
 
     while (!Freadyp(stream))
       for (idle = PORT_IDLE(stream); !NULLP(idle); idle = CDR(idle))
-	STk_C_apply(CAR(idle), 0);
+        STk_C_apply(CAR(idle), 0);
   }
 
   for ( ; ; ) {
@@ -246,18 +302,18 @@ static Inline int Fread(void *stream, void *buff, int count)
      return copied + read(PORT_FD(stream), buff, count);
 }
 
-static int Fclose(void *stream)		/* File version */
+static int Fclose(void *stream)         /* File version */
 {
   int ret = flush_buffer(stream);
 
   return (ret == EOF) ? EOF : close(PORT_FD(stream));
 }
 
-static int Fclose_pipe(void *stream)	/* pipe version (used by "| cmd" files */
+static int Fclose_pipe(void *stream)    /* pipe version (used by "| cmd" files */
 {
   int ret = flush_buffer(stream);
 
-  return (ret == EOF) ? EOF : pclose(PORT_FILE(stream));
+  return (ret == EOF) ? EOF : fd_pclose(PORT_FD(stream), PORT_PID(stream));
 }
 
 
@@ -276,15 +332,15 @@ static Inline int Fputc(int c, void *stream)
       *PORT_PTR(stream)++ = (unsigned char) c;
       PORT_CNT(stream)   += 1;
       if (c == '\n' && PORT_STREAM_FLAGS(stream) & STK_IOLBF) {
-	if (flush_buffer(stream) != 0)
-	  return EOF;
+        if (flush_buffer(stream) != 0)
+          return EOF;
       }
       break;
     }
     else {
       /* buffer is full */
       if (flush_buffer(stream) != 0)
-	return EOF;
+        return EOF;
     }
   }
   return ret;
@@ -321,19 +377,19 @@ static Inline int Fnputs(void *stream, char *s, int len)
       free = PORT_BUFSIZE(stream) - PORT_CNT(stream);
 
       if (len <= free) {
-	memcpy(PORT_PTR(stream), s, len);
-	PORT_PTR(stream) += len;
-	PORT_CNT(stream) += len;
-	res = count;
-	break;
+        memcpy(PORT_PTR(stream), s, len);
+        PORT_PTR(stream) += len;
+        PORT_CNT(stream) += len;
+        res = count;
+        break;
       } else {
-	memcpy(PORT_PTR(stream), s, free);
-	PORT_PTR(stream) += free;
-	PORT_CNT(stream) += free;
-	s += free;
-	len -= free;
-	if (flush_buffer(stream) != 0)
-	  { res = EOF; break; }
+        memcpy(PORT_PTR(stream), s, free);
+        PORT_PTR(stream) += free;
+        PORT_CNT(stream) += free;
+        s += free;
+        len -= free;
+        if (flush_buffer(stream) != 0)
+          { res = EOF; break; }
       }
     }
   }
@@ -358,25 +414,25 @@ static Inline int Fputstring(SCM s, void *stream)
 }
 
 
-static int Inline Fflush(void *stream)
+static Inline int  Fflush(void *stream)
 {
-  return flush_buffer(stream);		/* Problem if file opened for reading */
+  return flush_buffer(stream);          /* Problem if file opened for reading */
 }
 
 
-static void fport_print(SCM obj, SCM port) 	/* Generic printing of file ports */
+static void fport_print(SCM obj, SCM port)      /* Generic printing of file ports */
 {
   char buffer[MAX_PATH_LENGTH + 40];
 
-  sprintf(buffer, "#[%s-port '%s' (%d)%s]",
-	  IPORTP(obj) ? "input" : "output",
-	  PORT_FNAME(obj), PORT_FD(PORT_STREAM(obj)),
-	  PORT_IS_CLOSEDP(obj) ? " (closed)" : "");
+  snprintf(buffer, sizeof(buffer), "#[%s-port '%s' (%d)%s]",
+           IPORTP(obj) ? "input" : "output",
+           PORT_FNAME(obj), PORT_FD(PORT_STREAM(obj)),
+           PORT_IS_CLOSEDP(obj) ? " (closed)" : "");
   STk_puts(buffer, port);
 }
 
 
-static void fport_finalizer(struct port_obj *port)
+static void fport_finalizer(struct port_obj *port, void _UNUSED(*client_data))
 {
   if (!PORT_IS_CLOSEDP(port))
     /* Close the associated stream */
@@ -385,13 +441,12 @@ static void fport_finalizer(struct port_obj *port)
 
 
 static struct port_obj *
-make_fport(char *fname, FILE *f, int flags)
+make_fport(char *fname, int fd, int flags)
 {
   struct fstream  *fs = STk_must_malloc(sizeof(struct fstream));
-  int n, mode, fd;
+  int n, mode;
   SCM res;
 
-  fd = fileno(f);
   /* allocate buffer for file */
   if (isatty(fd) || (STk_interactive && fd < 2)) {
     n      = TTY_BUFSIZE;
@@ -402,46 +457,50 @@ make_fport(char *fname, FILE *f, int flags)
     mode = STK_IOFBF;
   }
 
-  /* keep the indication that file is opened in read in the steam part */
+  /* keep the indication that file is opened in read in the stream part */
   if (flags & (PORT_READ | PORT_RW)) mode |= STK_IOREAD;
 
+  /* Set the case sensitive bit */
+  if (STk_read_case_sensitive) flags |= PORT_CASE_SENSITIVE;
+
   /* Initialize the stream part */
-  PORT_BASE(fs)    	 = STk_must_malloc_atomic(n);
+  PORT_BASE(fs)          = STk_must_malloc_atomic(n);
   PORT_PTR(fs)           = PORT_BASE(fs);
-  PORT_CNT(fs)		 = 0;
-  PORT_BUFSIZE(fs) 	 = n;
+  PORT_CNT(fs)           = 0;
+  PORT_BUFSIZE(fs)       = n;
   PORT_STREAM_FLAGS(fs)  = mode;
-  PORT_FILE(fs)		 = f;
-  PORT_FD(fs)		 = fd;
-  PORT_REVENT(fs)	 = STk_false;
-  PORT_WEVENT(fs)	 = STk_false;
-  PORT_IDLE(fs)		 = STk_nil;
+  PORT_PID(fs)           = 0; /* will be changed if port is a pipe */
+  PORT_FD(fs)            = fd;
+  PORT_REVENT(fs)        = STk_false;
+  PORT_WEVENT(fs)        = STk_false;
+  PORT_IDLE(fs)          = STk_nil;
 
   /* Initialize now the port itsef */
   NEWCELL(res, port);
 
-  PORT_STREAM(res)	= fs;
-  PORT_FLAGS(res)	= flags;
-  PORT_UNGETC(res) 	= EOF;
-  PORT_FNAME(res)	= STk_strdup(fname);
-  PORT_LINE(res)	= 1;
-  PORT_POS(res)		= 0;
-  PORT_CLOSEHOOK(res)	= STk_false;
+  PORT_STREAM(res)      = fs;
+  PORT_FLAGS(res)       = flags;
+  PORT_UNGETC(res)      = EOF;
+  PORT_FNAME(res)       = STk_strdup(fname);
+  PORT_LINE(res)        = 1;
+  PORT_POS(res)         = 0;
+  PORT_KW_COL_POS(res)  = STk_keyword_colon_convention();
+  PORT_CLOSEHOOK(res)   = STk_false;
 
-  PORT_PRINT(res)	= fport_print;
-  PORT_RELEASE(res)	= unregister_port;
-  PORT_GETC(res)	= Fgetc;
-  PORT_READY(res)	= Freadyp;
-  PORT_EOFP(res)	= Feof;
-  PORT_CLOSE(res)	= (flags & PORT_IS_PIPE) ? Fclose_pipe: Fclose;
-  PORT_PUTC(res)	= Fputc;
-  PORT_PUTS(res)	= Fputs;
-  PORT_PUTSTRING(res)	= Fputstring;
-  PORT_NPUTS(res)	= Fnputs;
-  PORT_FLUSH(res)	= Fflush;
-  PORT_BREAD(res)	= Fread;
-  PORT_BWRITE(res)	= Fwrite;
-  PORT_SEEK(res)	= Fseek;
+  PORT_PRINT(res)       = fport_print;
+  PORT_RELEASE(res)     = unregister_port;
+  PORT_GETC(res)        = Fgetc;
+  PORT_READY(res)       = Freadyp;
+  PORT_EOFP(res)        = Feof;
+  PORT_CLOSE(res)       = (flags & PORT_IS_PIPE) ? Fclose_pipe: Fclose;
+  PORT_PUTC(res)        = Fputc;
+  PORT_PUTS(res)        = Fputs;
+  PORT_PUTSTRING(res)   = Fputstring;
+  PORT_NPUTS(res)       = Fnputs;
+  PORT_FLUSH(res)       = Fflush;
+  PORT_BREAD(res)       = Fread;
+  PORT_BWRITE(res)      = Fwrite;
+  PORT_SEEK(res)        = Fseek;
 
   /* Add a finalizer on file to close it when the GC frees it */
   STk_register_finalizer(res, fport_finalizer);
@@ -462,74 +521,94 @@ static char *convert_for_win32(char *mode)
    */
   switch (*mode) {
     case 'r': if (mode[1] == '\0') return "rb";
-      	      if (mode[1] == '+')  return "rb+";
-	      break;
+              if (mode[1] == '+')  return "rb+";
+              break;
     case 'w': if (mode[1] == '\0') return "wb";
-      	      if (mode[1] == '+')  return "wb+";
-	      break;
+              if (mode[1] == '+')  return "wb+";
+              break;
     case 'a': if (mode[1] == '\0') return "ab";
-      	      if (mode[1] == '+')  return "ab+";
-	      break;
+              if (mode[1] == '+')  return "ab+";
+              break;
   }
   return mode;
 }
 #endif
 
+static int convert_mode(char* mode) {
+  char first = *mode;
+
+  if (mode[1] == 'b')
+    mode++;
+
+  switch (first) {
+    case 'r':
+      if (mode[1] == '\0') return O_RDONLY;
+      if (mode[1] == '+')  return O_RDWR;
+      break;
+    case 'w':
+      if (mode[1] == '\0') return O_WRONLY | O_TRUNC | O_CREAT;
+      if (mode[1] == '+')  return O_RDWR   | O_TRUNC | O_CREAT;
+      break;
+    case 'a':
+      if (mode[1] == '\0') return O_WRONLY | O_CREAT | O_APPEND;
+      if (mode[1] == '+')  return O_RDWR   | O_CREAT | O_APPEND;
+      break;
+    default:
+      break;
+  }
+  STk_error("bad file opening mode %s", mode);
+  return 0;
+}
 
 
 static SCM open_file_port(SCM filename, char *mode, int flags, int error)
 {
-  FILE *f;
   char *full_name, *name;
+  SCM z;
+  int fd;
 
-  /* We use fopen (or popen) here to simplify problems with mode opening
-   * But since we don't use the buffer, we say that we work in non buffered
-   * mode
-   */
   name = STRING_CHARS(filename);
 
   if (strncmp(name, "| ", 2)) {
     full_name  = STk_expand_file_name(name);
     flags     |= PORT_IS_FILE;
 
-    if ((f = fopen(full_name, mode)) == NULL) {
+    if ((fd = open(full_name, convert_mode(mode), 0666)) == -1) {
       if (error)
-	STk_error_file_name("could not open file ~S", filename);
+        STk_error_file_name("could not open file ~S", filename);
       else
-	return STk_false;
+        return STk_false;
     }
-  }
-  else {
+    if (*mode == 'a') lseek(fd, 0L, SEEK_END);
+    z = make_fport(full_name, fd, flags);
+  } else {
+    pid_t pid = 0; /* for the compiler */
+
     full_name  = name;
     flags     |= PORT_IS_PIPE;
-    if ((f = popen(name+1, mode)) == NULL) {
+    if ((fd = fd_popen(name+1, mode, &pid)) < 0) {
       if (error)
-	STk_error_file_name("could not create pipe for ~S",
-			    STk_Cstring2string(name+2));
+        STk_error_file_name("could not create pipe for ~S",
+                            STk_Cstring2string(name+2));
       else
-	return STk_false;
+        return STk_false;
     }
+    z = make_fport(full_name, fd, flags);
+    PORT_PID(PORT_STREAM(z)) = pid;
   }
 
-  /* Don't use (and allocate) a buffer for this file */
-  setvbuf(f, NULL, _IONBF, 0);
-  return (SCM) make_fport(full_name, f, flags);
+  return z;
 }
 
 
+
+
 SCM STk_fd2scheme_port(int fd, char *mode, char *identification)
 {
-  FILE *f;
   int flags;
 
-  f = fdopen(fd, mode);
-  if (!f) return (SCM) NULL;
-
-  /* Don't use (and allocate) a buffer for this file */
-  setvbuf(f, NULL, _IONBF, 0);
-
   flags = PORT_IS_FILE | ((*mode == 'r') ? PORT_READ : PORT_WRITE);
-  return (SCM) make_fport(identification, f, flags);
+  return (SCM) make_fport(identification, fd, flags);
 }
 
 
@@ -543,6 +622,8 @@ SCM STk_open_file(char *filename, char *mode)
     case 'w': type = (mode[1] == '+') ? PORT_RW : PORT_WRITE; break;
     default:  goto Error;
   }
+  type |= PORT_TEXTUAL;                     /* by default */
+
   return open_file_port(STk_Cstring2string(filename), mode, type, FALSE);
 Error:
   STk_panic("bad opening mode %s", mode);
@@ -552,7 +633,7 @@ Error:
 SCM STk_add_port_idle(SCM port, SCM idle_func)
 {
   PORT_IDLE(PORT_STREAM(port)) = STk_cons(idle_func,
-					  PORT_IDLE(PORT_STREAM(port)));
+                                          PORT_IDLE(PORT_STREAM(port)));
   return STk_void;
 }
 
@@ -590,7 +671,13 @@ doc>
 DEFINE_PRIMITIVE("open-input-file", open_input_file, subr1, (SCM filename))
 {
   if (!STRINGP(filename)) STk_error_bad_file_name(filename);
-  return open_file_port(filename, "r", PORT_READ, TRUE);
+  return open_file_port(filename, "r", PORT_READ|PORT_TEXTUAL, TRUE);
+}
+
+DEFINE_PRIMITIVE("open-binary-input-file", open_binary_ifile, subr1, (SCM filename))
+{
+  if (!STRINGP(filename)) STk_error_bad_file_name(filename);
+  return open_file_port(filename, "r", PORT_READ|PORT_BINARY, TRUE);
 }
 
 /*
@@ -610,7 +697,13 @@ doc>
 DEFINE_PRIMITIVE("open-output-file", open_output_file, subr1, (SCM filename))
 {
   if (!STRINGP(filename)) STk_error_bad_file_name(filename);
-  return open_file_port(filename, "w", PORT_WRITE, TRUE);
+  return open_file_port(filename, "w", PORT_WRITE|PORT_TEXTUAL, TRUE);
+}
+
+DEFINE_PRIMITIVE("open-binary-output-file", open_binary_ofile, subr1, (SCM filename))
+{
+  if (!STRINGP(filename)) STk_error_bad_file_name(filename);
+  return open_file_port(filename, "w", PORT_WRITE|PORT_BINARY, TRUE);
 }
 
 
@@ -660,8 +753,8 @@ DEFINE_PRIMITIVE("output-file-port?", output_fportp, subr1, (SCM port))
  * (item [|"a+"| to open file for reading and writing. The file is created
  * if it does not exist. The stream is positioned at the end of the file.])
  * )
- * If the file can be opened, |open-file| returns the port associated with
- * the given file, otherwise it returns |#f|. Here again, the ``magic''
+ * If the file can be opened, |open-file| returns the textual port associated
+ * with the given file, otherwise it returns |#f|. Here again, the ``magic''
  * string "@pipe " permits to open a pipe port (in this case mode can only be
  * |"r"| or |"w"|).
 doc>
@@ -680,7 +773,7 @@ DEFINE_PRIMITIVE("open-file", scheme_open_file, subr2, (SCM filename, SCM mode))
     case 'w': type = (STRING_CHARS(mode)[1] == '+') ? PORT_RW : PORT_WRITE; break;
     default:  goto Error;
   }
-  return open_file_port(filename, STRING_CHARS(mode), type, FALSE);
+  return open_file_port(filename, STRING_CHARS(mode), type | PORT_TEXTUAL, FALSE);
 Error:
   STk_error_bad_io_param("bad opening mode ~S", mode);
   return STk_void; /* for the compiler */
@@ -722,14 +815,14 @@ DEFINE_PRIMITIVE("%port-idle", port_idle, subr12, (SCM port, SCM val))
 }
 
 /*=============================================================================*\
- * 				Load
+ *                              Load
 \*=============================================================================*/
 
-#define FILE_IS_SOURCE		0
-#define FILE_IS_BCODE 		1
-#define FILE_IS_OBJECT		2
-#define FILE_IS_DIRECTORY	3
-#define FILE_DOES_NOT_EXISTS	4
+#define FILE_IS_SOURCE          0
+#define FILE_IS_BCODE           1
+#define FILE_IS_OBJECT          2
+#define FILE_IS_DIRECTORY       3
+#define FILE_DOES_NOT_EXISTS    4
 
 static int find_file_nature(SCM f)
 {
@@ -766,7 +859,7 @@ SCM STk_load_source_file(SCM f)
      * change while loading the file (with R5RS macro for instance, it
      * is the case)
      */
-    sexpr = STk_read_constant(f, STk_read_case_sensitive);
+    sexpr = STk_read_constant(f, PORT_CASE_SENSITIVEP(f));
     if (sexpr == STk_eof) break;
     eval  = STk_lookup(eval_symb, STk_current_module(), &ref, TRUE);
     STk_C_apply(eval, 1, sexpr);
@@ -816,7 +909,7 @@ DEFINE_PRIMITIVE("load", scheme_load, subr1, (SCM filename))
 {
   if (!STRINGP(filename)) STk_error_bad_file_name(filename);
   if (load_file(filename) == STk_false)
-    STk_error_cannot_load(filename);
+    STk_error_file_name("cannot load ~S", filename);
   return STk_void;
 }
 
@@ -860,14 +953,14 @@ DEFINE_PRIMITIVE("%file-informations", file_informations, subr1, (SCM filename))
         case FILE_IS_SOURCE: str = "source"; break;
         case FILE_IS_BCODE:  str = "byte-code"; res = STk_read(f, TRUE); break;
         case FILE_IS_OBJECT: str = "object";
-			     res = STk_info_object_file(STRING_CHARS(filename));
-	  		     break;
+                             res = STk_info_object_file(STRING_CHARS(filename));
+                             break;
       }
       STk_close_port(f);
       /* Add the file nature to result */
       res = STk_cons(STk_makekey("nature"),
-		     STk_cons(STk_intern(str),
-			      res));
+                     STk_cons(STk_intern(str),
+                              res));
     }
   }
   return res;
@@ -878,12 +971,15 @@ int STk_init_fport(void)
 {
   vm_thread_t *vm = STk_get_current_vm();
 
-  STk_stdin  = vm->iport = (SCM) make_fport("*stdin*",  stdin,
-					    PORT_IS_FILE | PORT_READ);
-  STk_stdout = vm->oport = (SCM) make_fport("*stdout*", stdout,
-					    PORT_IS_FILE | PORT_WRITE);
-  STk_stderr = vm->eport = (SCM) make_fport("*stderr*", stderr,
-					    PORT_IS_FILE | PORT_WRITE);
+  STk_stdin  = vm->iport = (SCM) make_fport("*stdin*", 0,
+                                            PORT_IS_FILE | PORT_READ |
+                                            PORT_BINARY | PORT_TEXTUAL);
+  STk_stdout = vm->oport = (SCM) make_fport("*stdout*", 1,
+                                            PORT_IS_FILE | PORT_WRITE |
+                                            PORT_BINARY | PORT_TEXTUAL);
+  STk_stderr = vm->eport = (SCM) make_fport("*stderr*", 2,
+                                            PORT_IS_FILE | PORT_WRITE |
+                                            PORT_BINARY | PORT_TEXTUAL);
 
   ADD_PRIMITIVE(scheme_load);
   ADD_PRIMITIVE(scheme_try_load);
@@ -891,6 +987,8 @@ int STk_init_fport(void)
 
   ADD_PRIMITIVE(open_input_file);
   ADD_PRIMITIVE(open_output_file);
+  ADD_PRIMITIVE(open_binary_ifile);
+  ADD_PRIMITIVE(open_binary_ofile);
 
   ADD_PRIMITIVE(input_fportp);
   ADD_PRIMITIVE(output_fportp);
