@@ -1,21 +1,27 @@
 /*
- * p r o c e s s . c 		-- Access to processes from STklos
+ * p r o c e s s . c            -- Access to processes from STklos
  *
- * Copyright © 1994-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1994-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
- * Permission to use, copy, modify, distribute,and license this
- * software and its documentation for any purpose is hereby granted,
- * provided that existing copyright notices are retained in all
- * copies and that this notice is included verbatim in any
- * distributions.  No written agreement, license, or royalty fee is
- * required for any of the authorized uses.
- * This software is provided ``AS IS'' without express or implied
- * warranty.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ * USA.
  *
  *            Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??-???-1994 ??:??
- * Last file update: 27-May-2011 22:36 (eg)
+ * Last file update: 27-Aug-2021 20:36 (eg)
  *
  * Code for Win32 conributed by (Paul Anderson <paul@grammatech.com> and
  * Sarah Calvo <sarah@grammatech.com>) has been deleted for now. It should be
@@ -34,38 +40,39 @@
 #include <fcntl.h>
 #include <sys/param.h>
 #include <sys/wait.h>
+#include <sys/resource.h>
 #include <signal.h>
 #include "fport.h"
 
 /*
  * Data
  */
-static char *stdStreams[3] = {		/* Used for messages */
+static char *stdStreams[3] = {          /* Used for messages */
   "input",
   "output",
   "error",
 };
 
 
-#define MAX_PROC_NUM      40 		/* (simultaneous processes) enough? */
+#define MAX_PROC_NUM      40            /* (simultaneous processes) enough? */
 
 struct process_obj {
   stk_header header;
   int pid;                      /* Process id */
-  SCM streams[3];		/* standard ports for the process */
-  int exited;			/* Process is terminated */
-  int exit_status;		/* Exit status of the processus */
-  int waited_on;		/* non zero if the process is being
-   				   waited on by a waitpid(..,..,0) */
+  SCM streams[3];               /* standard ports for the process */
+  int exited;                   /* Process is terminated */
+  int exit_status;              /* Exit status of the processus */
+  int waited_on;                /* non zero if the process is being
+                                   waited on by a waitpid(..,..,0) */
 };
 
 
-#define PROCESS_PID(p)		(((struct process_obj *) (p))->pid)
-#define PROCESS_STREAMS(p)	(((struct process_obj *) (p))->streams)
-#define PROCESS_EXITED(p)	(((struct process_obj *) (p))->exited)
-#define PROCESS_STATUS(p)	(((struct process_obj *) (p))->exit_status)
-#define PROCESS_WAITED(p)	(((struct process_obj *) (p))->waited_on)
-#define PROCESSP(p)		(BOXED_TYPE_EQ((p), tc_process))
+#define PROCESS_PID(p)          (((struct process_obj *) (p))->pid)
+#define PROCESS_STREAMS(p)      (((struct process_obj *) (p))->streams)
+#define PROCESS_EXITED(p)       (((struct process_obj *) (p))->exited)
+#define PROCESS_STATUS(p)       (((struct process_obj *) (p))->exit_status)
+#define PROCESS_WAITED(p)       (((struct process_obj *) (p))->waited_on)
+#define PROCESSP(p)             (BOXED_TYPE_EQ((p), tc_process))
 
 static SCM all_processes = STk_nil;
 
@@ -74,12 +81,12 @@ static SCM all_processes = STk_nil;
 #endif
 
 #ifdef USE_SIGCHLD
-#  define PURGE_PROCESS_TABLE()				    /* Nothing to do */
+#  define PURGE_PROCESS_TABLE()                             /* Nothing to do */
 #else
 #  define PURGE_PROCESS_TABLE() process_terminate_handler(0)/* Simulate a SIGCHLD */
 #endif
 
-MUT_DECL(process_table_mutex);
+MUT_DECL(process_table_mutex)
 
 /******************************************************************************/
 
@@ -105,19 +112,19 @@ static int process_alivep(SCM process)
        return TRUE;
      else
        if (res == PROCESS_PID(process)) {
-	 /* process has terminated and we must save this information */
-	 PROCESS_EXITED(process) = TRUE;
-	 PROCESS_STATUS(process) = info;
-	 return FALSE;
+         /* process has terminated and we must save this information */
+         PROCESS_EXITED(process) = TRUE;
+         PROCESS_STATUS(process) = info;
+         return FALSE;
        } else {
-	 /* might not have found process because we've already waited for it */
-	 /* if so, then status has already been updated */
-	 return FALSE;
+         /* might not have found process because we've already waited for it */
+         /* if so, then status has already been updated */
+         return FALSE;
        }
    }
 }
 
-static void process_terminate_handler(int sig) /* called when a child dies */
+static void process_terminate_handler(int _UNUSED(sig)) /* called when a child dies */
 {
   SCM prev, l;
 
@@ -134,9 +141,9 @@ static void process_terminate_handler(int sig) /* called when a child dies */
     if (!process_alivep(CAR(l))) {
       /* Process died. delete it from the list */
       if (l == all_processes)
-	all_processes = CDR(l);
+        all_processes = CDR(l);
       else
-	CDR(prev) = CDR(l);
+        CDR(prev) = CDR(l);
     }
   }
   MUT_UNLOCK(process_table_mutex);
@@ -216,7 +223,7 @@ static char *maybe_redirect_input(SCM in, int pipes[3][2])
 }
 
 static char *maybe_redirect_output(SCM out, int index, int pipes[3][2],
-				   char *input, char *output)
+                                   char *input, char *output)
 {
   char *name = "";
   int fd;
@@ -256,16 +263,16 @@ static char *maybe_redirect_output(SCM out, int index, int pipes[3][2],
 
 /*===========================================================================*\
  *
- * 		Implementation of run-process and fork for Unix
+ *              Implementation of run-process and fork for Unix
  *
 \*==========================================================================*/
 
 DEFINE_PRIMITIVE("%run-process", run_process, subr4,
-		 (SCM redirections, SCM do_wait, SCM do_fork, SCM args))
+                 (SCM redirections, SCM do_wait, SCM do_fork, SCM args))
 {
   SCM z, l;
   char **argv;
-  char *in_name, *out_name, *err_name;
+  char *in_name, *out_name; // *err_name;
   int i, len;
   int pipes[3][2];
   SCM *redir;
@@ -278,7 +285,7 @@ DEFINE_PRIMITIVE("%run-process", run_process, subr4,
   redir = VECTOR_DATA(redirections);
 
   /* Build an argv array for exec system call */
-  len = STk_int_length(args);		  /* //FIXME:  Pas traité le rsh */
+  len = STk_int_length(args);             /* //FIXME:  Pas traité le rsh */
   if (len < 0)
     STk_error("bad argument list ~S", args);
   argv = STk_must_malloc((len + 3) * sizeof(char *));
@@ -292,7 +299,7 @@ DEFINE_PRIMITIVE("%run-process", run_process, subr4,
   /* Do (eventually) redirections */
   in_name  = maybe_redirect_input (redir[0], pipes);
   out_name = maybe_redirect_output(redir[1], 1, pipes, in_name, "");
-  err_name = maybe_redirect_output(redir[2], 2, pipes, in_name, out_name);
+  maybe_redirect_output(redir[2], 2, pipes, in_name, out_name);
 
   /* Build a process object */
   z   = make_process();
@@ -301,70 +308,77 @@ DEFINE_PRIMITIVE("%run-process", run_process, subr4,
   pid = (do_fork == STk_false) ? 0 : fork();
   switch (pid) {
     case -1:  close_all_files(pipes);
-	      STk_error("cannot create a new process");
-	      break;
+              STk_error("cannot create a new process");
+              break;
     case 0:  /* CHILD */
-      	     for(i = 0; i < 3; i++) {
- 	       if (STRINGP(redir[i])) {
-		 /* Redirection in a file */
- 		 dup2(pipes[i][0], i);
- 		 close(pipes[i][0]);
- 	       } else if (is_pipe_p(redir[i])) {
-		 /* Redirection in a pipe */
-		 dup2(pipes[i][(i==0)? 0 : 1], i);
-		 close(pipes[i][0]);
-		 close(pipes[i][1]);
-	       } else if (PORTP(redir[i])) {
-		 int fd= PORT_FD(PORT_STREAM(redir[i]));
+             for(i = 0; i < 3; i++) {
+               if (STRINGP(redir[i])) {
+                 /* Redirection in a file */
+                 dup2(pipes[i][0], i);
+                 close(pipes[i][0]);
+               } else if (is_pipe_p(redir[i])) {
+                 /* Redirection in a pipe */
+                 dup2(pipes[i][(i==0)? 0 : 1], i);
+                 close(pipes[i][0]);
+                 close(pipes[i][1]);
+               } else if (PORTP(redir[i])) {
+                 int fd= PORT_FD(PORT_STREAM(redir[i]));
 
-		 dup2(fd, i);
-		 close(fd);
-	       }
- 	     }
+                 dup2(fd, i);
+                 close(fd);
+               }
+             }
 
-	     /* close all remaining files */
-	     for(i = 3; i < NOFILE; i++) close(i);
+             /* close all remaining files */
+             {
+               struct rlimit rl;
 
- 	     /*  And then, EXEC'ing...  */
-  	     execvp(*argv, argv);
+               if (getrlimit(RLIMIT_NOFILE, &rl) == 0)
+                 for(i = 3; i < (int) rl.rlim_cur; i++) close(i);
+               else
+                 STk_warning("run-process: cannot close file descriptors > 2");
+             }
 
- 	     /* Cannot exec if we are here */
- 	     STk_fprintf(STk_current_error_port(), "**** Cannot  exec %s!\n", *argv);
- 	     _exit(1);
+             /*  And then, EXEC'ing...  */
+             execvp(*argv, argv);
+
+             /* Cannot exec if we are here */
+             STk_fprintf(STk_current_error_port(), "**** Cannot  exec %s!\n", *argv);
+             _exit(1);
     default: /* PARENT */
-      	     PROCESS_PID(z) = pid;
-       	     for(i = 0; i < 3; i++) {
- 	       if (STRINGP(redir[i]))
- 		 /* Redirection in a file */
- 		 close(pipes[i][0]);
- 	       else if (is_pipe_p(redir[i])) {
-		 /* Redirection in a pipe */
-		 SCM port;
-		 char buffer[100];
+             PROCESS_PID(z) = pid;
+             for(i = 0; i < 3; i++) {
+               if (STRINGP(redir[i]))
+                 /* Redirection in a file */
+                 close(pipes[i][0]);
+               else if (is_pipe_p(redir[i])) {
+                 /* Redirection in a pipe */
+                 SCM port;
+                 char buffer[100];
 
-		 close(pipes[i][i == 0 ? 0 : 1]);
-		 /* Make a new file descriptor to access the pipe */
-		 sprintf(buffer, "pipe-%s-%d", stdStreams[i], pid);
-		 port = (i == 0) ?
-		   		STk_fd2scheme_port(pipes[i][1], "w", buffer) :
-		   		STk_fd2scheme_port(pipes[i][0], "r", buffer);
-		 if (!port) {
-		   close_all_files(pipes);
-		   STk_error("cannot reopen pipe %d for process %d", i, pid);
-		 }
-		 PROCESS_STREAMS(z)[i] = port;
-	       } else if (PORTP(redir[i])) {
-		 /* Redirection in a port */
-		 PROCESS_STREAMS(z)[i]= redir[i];
-	       }
+                 close(pipes[i][i == 0 ? 0 : 1]);
+                 /* Make a new file descriptor to access the pipe */
+                 snprintf(buffer, sizeof(buffer), "pipe-%s-%d", stdStreams[i], pid);
+                 port = (i == 0) ?
+                                STk_fd2scheme_port(pipes[i][1], "w", buffer) :
+                                STk_fd2scheme_port(pipes[i][0], "r", buffer);
+                 if (!port) {
+                   close_all_files(pipes);
+                   STk_error("cannot reopen pipe %d for process %d", i, pid);
+                 }
+                 PROCESS_STREAMS(z)[i] = port;
+               } else if (PORTP(redir[i])) {
+                 /* Redirection in a port */
+                 PROCESS_STREAMS(z)[i]= redir[i];
+               }
 
-	       if (do_wait != STk_false) {
-		 PROCESS_WAITED(z) = 1;
-		 waitpid(pid, &(PROCESS_STATUS(z)), 0);
-		 PROCESS_WAITED(z) = 0;
-		 PROCESS_EXITED(z) = TRUE;
-	       }
-	     }
+               if (do_wait != STk_false) {
+                 PROCESS_WAITED(z) = 1;
+                 waitpid(pid, &(PROCESS_STATUS(z)), 0);
+                 PROCESS_WAITED(z) = 0;
+                 PROCESS_EXITED(z) = TRUE;
+               }
+             }
   }
   /* Chain new process in the list of all process */
   MUT_LOCK(process_table_mutex);
@@ -373,7 +387,7 @@ DEFINE_PRIMITIVE("%run-process", run_process, subr4,
   return z;
 }
 
-		/* ======================================== */
+                /* ======================================== */
 
 /*
 <doc EXT fork
@@ -404,15 +418,16 @@ DEFINE_PRIMITIVE("fork", fork, subr01, (SCM thunk))
   /* Fork another process */
   pid = fork();
   switch (pid) {
-    case -1: 						      /* ERROR */
+    case -1:                                                  /* ERROR */
       STk_error("cannot create a new process");
-    case 0:  						      /* CHILD */
+      /* FALLTHRU */
+    case 0:                                                   /* CHILD */
       if (thunk) {
-	STk_C_apply(thunk, 0);
-	STk_exit(0);
+        STk_C_apply(thunk, 0);
+        STk_exit(0);
       }
       return STk_false;
-    default: 						      /* PARENT */
+    default:                                                  /* PARENT */
       z  = make_process();
       PROCESS_PID(z) = pid;
       /* Chain new process in the list of all process */
@@ -452,88 +467,88 @@ DEFINE_PRIMITIVE("fork", fork, subr01, (SCM thunk))
 //   pid = fork();
 //   switch (pid) {
 //     case -1:  close_all_files(pipes);
-// 	      STk_error("cannot create a new process");
+//            STk_error("cannot create a new process");
 //     case 0:
 //       /* CHILD */
 //       {
-// 	char buffer[100], *str;
+//      char buffer[100], *str;
 //
-// 	for(i = 0; i < 3; i++) {
-// 	  if (STRINGP(redir[i])) {
-// 	    /* Redirection in a file */
-// 	    dup2(pipes[i][0], i);
-// 	    close(pipes[i][0]);
+//      for(i = 0; i < 3; i++) {
+//        if (STRINGP(redir[i])) {
+//          /* Redirection in a file */
+//          dup2(pipes[i][0], i);
+//          close(pipes[i][0]);
 //
-// 	    str = STk_expand_file_name(STRING_CHARS(redir[i]));
-// 	    switch (i) {
-// 	    case 0:   /* FIXME: STk_close_port(STk_curr_iport);*/
-// 		      STk_curr_iport = STk_fd2scheme_port(i, "r", str);
-// 		      break;
-// 	      case 1: STk_close_port(STk_curr_oport);
-// 		      STk_curr_oport = STk_fd2scheme_port(i, "w", str);
-// 		      break;
-// 	      case 2: STk_close_port(STk_curr_eport);
-// 		      STk_curr_eport = STk_fd2scheme_port(i, "w", str);
-// 		      break;
-// 	    }
-// 	  } else if (is_pipe_p(redir[i])) {
-// 	    /* Redirection in a pipe */
-// 	    dup2(pipes[i][i==0? 0 : 1], i);
-// 	    close(pipes[i][0]);
-// 	    close(pipes[i][1]);
+//          str = STk_expand_file_name(STRING_CHARS(redir[i]));
+//          switch (i) {
+//          case 0:   /* FIXME: STk_close_port(STk_curr_iport);*/
+//                    STk_curr_iport = STk_fd2scheme_port(i, "r", str);
+//                    break;
+//            case 1: STk_close_port(STk_curr_oport);
+//                    STk_curr_oport = STk_fd2scheme_port(i, "w", str);
+//                    break;
+//            case 2: STk_close_port(STk_curr_eport);
+//                    STk_curr_eport = STk_fd2scheme_port(i, "w", str);
+//                    break;
+//          }
+//        } else if (is_pipe_p(redir[i])) {
+//          /* Redirection in a pipe */
+//          dup2(pipes[i][i==0? 0 : 1], i);
+//          close(pipes[i][0]);
+//          close(pipes[i][1]);
 //
-// 	    sprintf(buffer, "pipe-%s-%d", stdStreams[i], getpid());
-// 	    switch (i) {
-// 	      case 0: /* FIXME: STk_close_port(STk_curr_iport); */
-// 		      STk_curr_iport = STk_fd2scheme_port(i, "r", buffer);
-// 		      break;
-// 	      case 1: STk_close_port(STk_curr_oport);
-// 		      STk_curr_oport = STk_fd2scheme_port(i, "w", buffer);
-// 		      break;
-// 	      case 2: STk_close_port(STk_curr_eport);
-// 		      STk_curr_eport = STk_fd2scheme_port(i, "w", buffer);
-// 		      break;
-// 	    }
-// 	  }
-// 	}
+//          sprintf(buffer, "pipe-%s-%d", stdStreams[i], getpid());
+//          switch (i) {
+//            case 0: /* FIXME: STk_close_port(STk_curr_iport); */
+//                    STk_curr_iport = STk_fd2scheme_port(i, "r", buffer);
+//                    break;
+//            case 1: STk_close_port(STk_curr_oport);
+//                    STk_curr_oport = STk_fd2scheme_port(i, "w", buffer);
+//                    break;
+//            case 2: STk_close_port(STk_curr_eport);
+//                    STk_curr_eport = STk_fd2scheme_port(i, "w", buffer);
+//                    break;
+//          }
+//        }
+//      }
 //
-// 	/* close all remaining files */
-// 	for(i = 3; i < NOFILE; i++) close(i);
-// 	return STk_false;
+//      /* close all remaining files */
+//      for(i = 3; i < NOFILE; i++) close(i);
+//      return STk_false;
 //       }
 //     default:
 //       {
-// 	/* PARENT */
-// 	z  = make_process();
-// 	PROCESS_PID(z) = pid;
+//      /* PARENT */
+//      z  = make_process();
+//      PROCESS_PID(z) = pid;
 //
-// 	/* Chain new process in the list of all process */
-// 	all_processes = STk_cons(z, all_processes);
+//      /* Chain new process in the list of all process */
+//      all_processes = STk_cons(z, all_processes);
 //
-// 	/* Do the redirections if necessary */
-// 	for(i = 0; i < 3; i++) {
-// 	  if (STRINGP(redir[i]))
-// 	    /* Redirection in a file */
-// 	    close(pipes[i][0]);
-// 	  else if (is_pipe_p(redir[i])) {
-// 	    /* Redirection in a pipe */
-// 	    char buffer[100];
+//      /* Do the redirections if necessary */
+//      for(i = 0; i < 3; i++) {
+//        if (STRINGP(redir[i]))
+//          /* Redirection in a file */
+//          close(pipes[i][0]);
+//        else if (is_pipe_p(redir[i])) {
+//          /* Redirection in a pipe */
+//          char buffer[100];
 //
-// 	    switch (i) {
-// 	      case 0:
-// 		dup2(pipes[i][1], 1);
-// 		//STk_close_port(STk_curr_oport);
-// 		sprintf(buffer, "pipe-%s-%d", stdStreams[1], pid);
-// 		STk_curr_oport = STk_fd2scheme_port(1, "w", buffer);
-// 		break;
-// 	    case 1:
-// 	    case 2: ;
-// 	    }
-// 	    close(pipes[i][0]);
-// 	    close(pipes[i][1]);
-// 	  }
-// 	}
-// 	return z;
+//          switch (i) {
+//            case 0:
+//              dup2(pipes[i][1], 1);
+//              //STk_close_port(STk_curr_oport);
+//              sprintf(buffer, "pipe-%s-%d", stdStreams[1], pid);
+//              STk_curr_oport = STk_fd2scheme_port(1, "w", buffer);
+//              break;
+//          case 1:
+//          case 2: ;
+//          }
+//          close(pipes[i][0]);
+//          close(pipes[i][1]);
+//        }
+//      }
+//      return z;
 //       }
 //   }
 //   return STk_false;
@@ -592,7 +607,7 @@ doc>
 DEFINE_PRIMITIVE("process-list", proc_list, subr0, (void))
 {
   PURGE_PROCESS_TABLE();
-  return STk_copy_tree(all_processes);
+  return STk_list_copy(all_processes);
 }
 
 
@@ -653,7 +668,7 @@ DEFINE_PRIMITIVE("process-wait", proc_wait, subr1, (SCM proc))
     res = waitpid(PROCESS_PID(proc), &info, 0);
     if (res == PROCESS_PID(proc)) {
       PROCESS_STATUS(proc) = info;
-      ret_val =  STk_true;;
+      ret_val =  STk_true;
     }
 
     PROCESS_WAITED(proc) = 0;
@@ -727,19 +742,19 @@ DEFINE_PRIMITIVE("process-signal", proc_signal, subr2, (SCM proc, SCM sig))
 
 
 /******************************************************************************/
-static void print_process(SCM p, SCM port, int mode)
+static void print_process(SCM p, SCM port, int _UNUSED(mode))
 {
   char buffer[100];
 
-  sprintf(buffer, "#<process PID=%d>", PROCESS_PID(p));
+  snprintf(buffer, sizeof(buffer), "#<process PID=%d>", PROCESS_PID(p));
   STk_puts(buffer, port);
 }
 
 
 
 static struct extended_type_descr xtype_process = {
-  "process",
-  print_process
+  .name  = "process",
+  .print = print_process
 };
 
 
