@@ -1,8 +1,8 @@
 /*
  *
- * s t r . c				-- Strings management
+ * s t r . c                            -- Strings management
  *
- * Copyright © 1993-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1993-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,16 +22,18 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??????
- * Last file update:  9-Sep-2011 15:37 (eg)
+ * Last file update:  6-Sep-2021 18:44 (eg)
  */
 
 #include <ctype.h>
-#include <wctype.h>
 #include "stklos.h"
 
 
+extern SCM STk_make_bytevector_from_string(char *str, long len);
+
+
 /* min size added to a string when reallocated in a string-set! */
-#define UTF8_STRING_INCR	8
+#define UTF8_STRING_INCR        8
 
 
 /*
@@ -39,6 +41,11 @@
  *
  */
 
+static void error_bad_index(SCM s)
+{
+  STk_error("bad index ~s", s);
+}
+
 static void error_bad_string(SCM s)
 {
   STk_error("bad string ~s", s);
@@ -49,6 +56,11 @@ static void error_bad_character(SCM s)
   STk_error("bad character ~S", s);
 }
 
+static void error_bad_string_or_character(SCM s)
+{
+  STk_error("bad string or character ~S", s);
+}
+
 static void error_change_const_string(SCM s)
 {
   STk_error("changing the constant string ~s is not allowed", s);
@@ -82,9 +94,9 @@ static int stringcomp(SCM s1, SCM s2)
 
     while ((str1 < end1) && (str2 < end2)) {
       if ((str1 = STk_utf8_grab_char(str1, &ch1)) == NULL)
-	error_bad_sequence(STRING_CHARS(s1));
+        error_bad_sequence(STRING_CHARS(s1));
       if ((str2 = STk_utf8_grab_char(str2, &ch2)) == NULL)
-	error_bad_sequence(STRING_CHARS(s2));
+        error_bad_sequence(STRING_CHARS(s2));
 
       if (ch1 != ch2) return ch1 - ch2;
     }
@@ -96,9 +108,9 @@ static int stringcomp(SCM s1, SCM s2)
     register int l1, l2;
 
     for (l1=STRING_SIZE(s1), str1=STRING_CHARS(s1),
-	 l2=STRING_SIZE(s2),str2=STRING_CHARS(s2);
-	 l1 && l2;
-	 l1--, str1++, l2--, str2++)
+         l2=STRING_SIZE(s2),str2=STRING_CHARS(s2);
+         l1 && l2;
+         l1--, str1++, l2--, str2++)
       if (*str1 != *str2) return ((unsigned char) *str1 - (unsigned char) *str2);
 
     /* l1 == 0 || l2 == 0 */
@@ -123,12 +135,16 @@ static int stringcompi(SCM s1, SCM s2)
     str2 = STRING_CHARS(s2); end2 = str2 + STRING_SIZE(s2);
 
     while ((str1 < end1) && (str2 < end2)) {
+      char c1low, c2low;
       if ((str1 = STk_utf8_grab_char(str1, &ch1)) == NULL)
-	error_bad_sequence(STRING_CHARS(s1));
+        error_bad_sequence(STRING_CHARS(s1));
       if ((str2 = STk_utf8_grab_char(str2, &ch2)) == NULL)
-	error_bad_sequence(STRING_CHARS(s2));
+        error_bad_sequence(STRING_CHARS(s2));
 
-      if (towlower(ch1) != towlower(ch2)) return towlower(ch1) - towlower(ch2);
+      c1low = STk_to_lower(ch1);
+      c2low = STk_to_lower(ch2);
+
+      if (c1low != c2low) return c1low - c2low;
     }
 
     /* str1 < end1 || str2 < end2 */
@@ -138,11 +154,11 @@ static int stringcompi(SCM s1, SCM s2)
     register int l1, l2;
 
     for (l1=STRING_SIZE(s1), str1=STRING_CHARS(s1),
-	 l2=STRING_SIZE(s2),str2=STRING_CHARS(s2);
-	 l1 && l2;
-	 l1--, str1++, l2--, str2++)
+         l2=STRING_SIZE(s2),str2=STRING_CHARS(s2);
+         l1 && l2;
+         l1--, str1++, l2--, str2++)
       if (tolower(*str1) != tolower(*str2))
-	return (tolower(*str1) - tolower(*str2));
+        return (tolower(*str1) - tolower(*str2));
 
     /* l1 == 0 || l2 == 0 */
     return l1 ? +1 : (l2 ? -1 : 0);
@@ -150,36 +166,36 @@ static int stringcompi(SCM s1, SCM s2)
 }
 
 
-static SCM control_index(int argc, SCM *argv, int *pstart, int *pend)
+static SCM control_index(int argc, SCM *argv, long *pstart, long *pend)
 {
-  SCM s = NULL;
+  SCM s = STk_void;   /* value chosen to avoid a warning of gcc static analysis */
   long len, start=0, end=-1;
 
   /* Controling number of arguments */
   switch (argc) {
-  case 1: s = argv[0]; break;
-  case 2: s = argv[0]; start = STk_integer_value(argv[-1]); break;
-  case 3: s = argv[0]; start = STk_integer_value(argv[-1]);
-	  end = STk_integer_value(argv[-2]); break;
-  default: STk_error("incorrect number of argument (%d)", argc);
+  case 1:  s = argv[0]; break;
+  case 2:  s = argv[0]; start = STk_integer_value(argv[-1]); break;
+  case 3:  s = argv[0]; start = STk_integer_value(argv[-1]);
+           end = STk_integer_value(argv[-2]); break;
+  default: STk_error("incorrect number of arguments (%d)", argc);
   }
 
   /* Controlling s */
   if (!STRINGP(s)) error_bad_string(s);
   len = STRING_LENGTH(s);
 
-  /* Controling start index */
+  /* Controlling start index */
   if (start == LONG_MIN || start < 0 || start > len)
     /* argc cannot be 1 (start would be 0) */
-    STk_error("bad starting index ~S", argv[(argc==2) ? 0 : -1]);
+    STk_error("bad starting index ~S", argv[-1]);
 
-  /* controling end index */
+  /* Controlling end index */
   if (end == -1)
     end = len;
   else
     if (end == LONG_MIN  || end < 0 || end > len)
       /* We have an end index ==> argc = 3 */
-      STk_error("bad ending index ~S", argv[0]);
+      STk_error("bad ending index ~S", argv[-2]);
 
   if (start > end)
     STk_error("low index is greater than high index");
@@ -190,7 +206,7 @@ static SCM control_index(int argc, SCM *argv, int *pstart, int *pend)
   return s;
 }
 
-static uint32_t *string2int(char *s, int len, int *utf8_len, wint_t(*func)(wint_t))
+static uint32_t *string2int(char *s, int len, int *utf8_len, uint32_t(*func)(uint32_t))
 {
   uint32_t ch, *tmp, *buff = STk_must_malloc_atomic(len * sizeof(uint32_t));
   int space = 0;
@@ -232,6 +248,28 @@ static void copy_array(uint32_t *buff, int len, char* from)
     from += STk_char2utf8(*buff++, from);
 }
 
+static SCM make_substring(SCM string, long from, long to)
+{
+  /* WARNING: from and to must be checked by caller */
+  if (STRING_MONOBYTE(string))
+    return STk_makestring(to - from, STRING_CHARS(string)+from);
+  else {
+    /* multi-bytes string */
+    uint32_t c;
+    char *pfrom, *pto;
+    SCM z;
+
+    pto = pfrom = STk_utf8_index(STRING_CHARS(string), from, STRING_SIZE(string));
+
+    for ( ; from < to; from++)
+      pto = STk_utf8_grab_char(pto, &c);
+
+    z = STk_makestring(pto - pfrom, pfrom);
+    STRING_LENGTH(z) = STk_utf8_strlen(STRING_CHARS(z), pto-pfrom);
+    return z;
+  }
+}
+
 
 SCM STk_makestring(int len, char *init)
 {
@@ -242,15 +280,18 @@ SCM STk_makestring(int len, char *init)
   STRING_SPACE(z) = STRING_SIZE(z) = STRING_LENGTH(z) = len;
 
   if (init) {
+    /* use memcpy instead of str... here because init may eventually contain
+     * null characters.
+     */
     memcpy(STRING_CHARS(z), init, (size_t) len);
-    STRING_CHARS(z)[len] = '\0'; /* so that STRING_CHARS is compatible with C */
+    STRING_CHARS(z)[len] = '\0'; /* so that STRING_CHARS is ~ compatible with C */
 
     if (STk_use_utf8)
-      /* Eventually correct the length to be in charcaters instead of bytes */
+      /* Eventually correct the length to be in characters instead of bytes */
       STRING_LENGTH(z) = STk_utf8_strlen(STRING_CHARS(z), len);
   }
   else
-    bzero(STRING_CHARS(z), len+1);
+    memset(STRING_CHARS(z), '\0', len+1);
 
   return z;
 }
@@ -264,8 +305,8 @@ SCM STk_Cstring2string(char *str) /* Embed a C string in Scheme world  */
   NEWCELL(z, string);
   STRING_CHARS(z)  = STk_must_malloc_atomic(len + 1);
   STRING_SPACE(z)  = STRING_SIZE(z) = len;
-  STRING_LENGTH(z) = STk_use_utf8 ? STk_utf8_strlen(str, len): len;
-  strcpy(STRING_CHARS(z), str);
+  STRING_LENGTH(z) = STk_use_utf8 ? (size_t) STk_utf8_strlen(str, len): len;
+  snprintf(STRING_CHARS(z), len+1, "%s", str);
 
   return z;
 }
@@ -309,28 +350,28 @@ DEFINE_PRIMITIVE("make-string", make_string, subr12, (SCM len, SCM init_char))
       int c = CHARACTER_VAL(init_char);
 
       if (STk_use_utf8 && c >= 0x80) {
-	/* unicode character */
-	int n = STk_char2utf8(c, buff);
+        /* unicode character */
+        int n = STk_char2utf8(c, buff);
 
-	z = STk_makestring(k * n, NULL);
-	s = STRING_CHARS(z);
-	STRING_LENGTH(z) = k; /* incorrectly set to  k*n before */
+        z = STk_makestring(k * n, NULL);
+        s = STRING_CHARS(z);
+        STRING_LENGTH(z) = k; /* incorrectly set to  k*n before */
 
-	while (k--) {
-	  *s++ = buff[0];
-	  if (!buff[1]) continue;
-	  *s++ = buff[1];
-	  if (!buff[2]) continue;
-	  *s++ = buff[2];
-	  if (!buff[3]) continue;
-	  *s++ = buff[3];
-	}
+        while (k--) {
+          *s++ = buff[0];
+          if (!buff[1]) continue;
+          *s++ = buff[1];
+          if (!buff[2]) continue;
+          *s++ = buff[2];
+          if (!buff[3]) continue;
+          *s++ = buff[3];
+        }
       } else {
-	/* non unicode character */
-	z = STk_makestring(k, NULL);
-	s = STRING_CHARS(z);
+        /* non unicode character */
+        z = STk_makestring(k, NULL);
+        s = STRING_CHARS(z);
 
-	while (k--) *s++ = c;
+        while (k--) *s++ = c;
       }
     }
     return z;
@@ -361,13 +402,13 @@ DEFINE_PRIMITIVE("string", string, vsubr, (int argc, SCM* argv))
       space += STk_char2utf8(CHARACTER_VAL(argv[-i]), buff);
     else
       if (CHARACTER_VAL(argv[-i]) > 255)
-	STk_error("character ~S is too big", argv[-i]);
+        STk_error("character ~S is too big", argv[-i]);
       else
-	space += 1;
+        space += 1;
   }
 
   z = STk_makestring(space, NULL);
-  STRING_LENGTH(z) = argc;	/* correct the length */
+  STRING_LENGTH(z) = argc;      /* correct the length */
 
   /* copy element in newly allocated string */
   for (s=STRING_CHARS(z); argc--; argv--) {
@@ -410,12 +451,12 @@ DEFINE_PRIMITIVE("string-ref", string_ref, subr2, (SCM str, SCM index))
 {
   long k = STk_integer_value(index);
 
-  if (!STRINGP(str))			error_bad_string(str);
+  if (!STRINGP(str))                    error_bad_string(str);
   if (k < 0 || k >= STRING_LENGTH(str)) error_index_out_of_bound(str, index);
 
   if (!STk_use_utf8 || (STRING_SIZE(str) == STRING_LENGTH(str)))
     /* string doesn't contain multibytes chars */
-    return MAKE_CHARACTER(STRING_CHARS(str)[k]);
+    return MAKE_CHARACTER((unsigned char)STRING_CHARS(str)[k]);
   else {
     /* We have multibytes chars */
     uint32_t c;
@@ -451,15 +492,15 @@ DEFINE_PRIMITIVE("string-set!", string_set, subr3, (SCM str, SCM index, SCM valu
   long k = STk_integer_value(index);
   int cval;
 
-  if (!STRINGP(str))    	        error_bad_string(str);
+  if (!STRINGP(str))                    error_bad_string(str);
   if (BOXED_INFO(str) & STRING_CONST)   error_change_const_string(str);
-  if (!CHARACTERP(value))	        error_bad_character(value);
+  if (!CHARACTERP(value))               error_bad_character(value);
   if (k < 0 || k >= STRING_LENGTH(str)) error_index_out_of_bound(str, index);
 
 
   cval = CHARACTER_VAL(value);
   if (!STk_use_utf8 || (STRING_MONOBYTE(str) &&
-			(STk_utf8_char_bytes_needed(cval) == 1)))
+                        (STk_utf8_char_bytes_needed(cval) == 1)))
     /* string doesn't contain multibytes chars and value is mono byte */
     STRING_CHARS(str)[k] = cval;
   else {
@@ -473,29 +514,29 @@ DEFINE_PRIMITIVE("string-set!", string_set, subr3, (SCM str, SCM index, SCM valu
     if (old_char_sz < new_char_sz) {
       /* new character has a longer representation than the old one */
       if (STRING_LENGTH(str) + new_char_sz - old_char_sz >= STRING_SPACE(str)) {
-	/* not enough space; allocate some more bytes */
-	char *new = STk_must_malloc_atomic(STRING_SPACE(str) + UTF8_STRING_INCR + 1);
+        /* not enough space; allocate some more bytes */
+        char *new = STk_must_malloc_atomic(STRING_SPACE(str) + UTF8_STRING_INCR + 1);
 
-	memcpy(new, start, pos - start);
-	memcpy(new + (pos - start), buffer, new_char_sz);
-	memcpy(new + (pos - start) + new_char_sz,
-	       pos + old_char_sz,
-	       start + STRING_SIZE(str) - pos + old_char_sz);
-	STRING_CHARS(str)  = new;
-	STRING_SPACE(str) += UTF8_STRING_INCR;
+        memcpy(new, start, pos - start);
+        memcpy(new + (pos - start), buffer, new_char_sz);
+        memcpy(new + (pos - start) + new_char_sz,
+               pos + old_char_sz,
+               start + STRING_SIZE(str) - pos + old_char_sz);
+        STRING_CHARS(str)  = new;
+        STRING_SPACE(str) += UTF8_STRING_INCR;
       } else {
-	memmove(pos + new_char_sz,
-		pos + old_char_sz,
-		start + STRING_SIZE(str) - pos + old_char_sz);
-	memcpy(pos, buffer, new_char_sz);
+        memmove(pos + new_char_sz,
+                pos + old_char_sz,
+                start + STRING_SIZE(str) - pos + old_char_sz);
+        memcpy(pos, buffer, new_char_sz);
       }
       STRING_SIZE(str) += new_char_sz - old_char_sz;
     }
     else if (old_char_sz > new_char_sz) {
       /* old character has a longer representation than the old one */
       memmove(pos + new_char_sz,
-	      pos + old_char_sz,
-	      start + STRING_SIZE(str) - pos + old_char_sz);
+              pos + old_char_sz,
+              start + STRING_SIZE(str) - pos + old_char_sz);
       memcpy(pos, buffer, new_char_sz);
       STRING_SIZE(str) += new_char_sz - old_char_sz;
     }
@@ -510,26 +551,28 @@ DEFINE_PRIMITIVE("string-set!", string_set, subr3, (SCM str, SCM index, SCM valu
 }
 
 /*
-<doc string=? string-ci=?
- * (string=? string1 string2)
- * (string-ci=? string1 string2)
+<doc R57RS string=? string-ci=?
+ * (string=? string1 string2 ...)
+ * (string-ci=? string1 string2 ...)
  *
- * Returns |#t| if the two strings are the same length and contain the same
+ * Returns |#t| if all the strings are the same length and contain the same
  * characters in the same positions, otherwise returns |#f|. |String-ci=?|
  * treats upper and lower case letters as though they were the same character,
  * but |string=?| treats upper and lower case as distinct characters.
+ * @l
+ * ,(bold "Note"): R5RS version of these functions accept only two arguments.
 doc>
  */
 
 /*
-<doc  string<? string<=? string>? string>=? string-ci<? string-ci<=? string-ci>? string-ci>=?
- * (string<? string1 string2)
- * (string>? string1 string2)
- * (string<=? string1 string2)
- * (string>=? string1 string2)
- * (string-ci<? string1 string2)
- * (string-ci>? string1 string2)
- * (string-ci<=? string1 string2)
+<doc  R57RS string<? string<=? string>? string>=? string-ci<? string-ci<=? string-ci>? string-ci>=?
+ * (string<? string1 string2 ...)
+ * (string>? string1 string2 ...)
+ * (string<=? string1 string2 ...)
+ * (string>=? string1 string2 ...)
+ * (string-ci<? string1 string2 ...)
+ * (string-ci>? string1 string2 ...)
+ * (string-ci<=? string1 string2 ...)
  * (string-ci>=? string1 string2)
  *
  * These procedures are the lexicographic extensions to strings of the
@@ -538,7 +581,8 @@ doc>
  * characters. If two strings differ in length but are the same up to the
  * length of the shorter string, the shorter string is considered to be
  * lexicographically less than the longer string.
- *
+ * @l
+ * ,(bold "Note"): R5RS version of these functions accept only two arguments.
 doc>
  */
 
@@ -577,7 +621,7 @@ DEFINE_PRIMITIVE("string-ci>=?", strgei, subr2, (SCM s1, SCM s2))
 
 
 /*
-<doc  substring
+<doc substring
  * (substring string start end)
  *
  * |String| must be a string, and |start| and |end| must be exact integers
@@ -590,7 +634,6 @@ DEFINE_PRIMITIVE("string-ci>=?", strgei, subr2, (SCM s1, SCM s2))
  * index |end| (exclusive).
 doc>
  */
-
 DEFINE_PRIMITIVE("substring", substring, subr3, (SCM string, SCM start, SCM end))
 {
   long from, to;
@@ -603,25 +646,9 @@ DEFINE_PRIMITIVE("substring", substring, subr3, (SCM string, SCM start, SCM end)
   if (from == LONG_MIN) STk_error("bad lower index ~S", start);
   if (to   == LONG_MIN) STk_error("bad upper index ~S", end);
 
-  if (0 <= from && from <= to && to <= STRING_SIZE(string)) {
-    if (STRING_MONOBYTE(string))
-      return STk_makestring(to - from, STRING_CHARS(string)+from);
-    else {
-      /* multi-bytes string */
-      uint32_t c;
-      char *pfrom, *pto;
-      SCM z;
+  if (0 <= from && from <= to && to <= STRING_SIZE(string))
+    return make_substring(string, from, to);
 
-      pto = pfrom = STk_utf8_index(STRING_CHARS(string), from, STRING_SIZE(string));
-
-      for ( ; from < to; from++)
-	pto = STk_utf8_grab_char(pto, &c);
-
-      z = STk_makestring(pto - pfrom, pfrom);
-      STRING_LENGTH(z) = STk_utf8_strlen(STRING_CHARS(z), pto-pfrom);
-      return z;
-    }
-  }
   STk_error("index ~S or ~S incorrect", start, end);
   return STk_void; /* cannot occur */
 }
@@ -663,19 +690,246 @@ DEFINE_PRIMITIVE("string-append", string_append, vsubr, (int argc, SCM* argv))
   return z;
 }
 
-
 /*
-<doc  string->list list->string
- * (string->list string)
- * (list->string list)
+<doc EXT string-append!
+ * (string-append! string ...)
  *
- * |String->list| returns a newly allocated list of the characters that make
- * up the given string. |List->string| returns a newly allocated string
- * formed from the characters in the list |list|, which must be a list of
- * characters. |String->list| and |list->string| are inverses so far as
- * |equal?| is concerned.
+ * Extends string by appending each value (in order) to the end of string.
+ * A value can be a character or a string.
+ *
+ * It is guaranteed that string-append! will return the same object that
+ * was passed to it as first argument, whose size may be larger.
+ *
+ * ,(linebreak)
+ * ,(bold "Note:") This function is defined in SRFI-118.
 doc>
  */
+DEFINE_PRIMITIVE("string-append!", string_dappend, vsubr, (int argc, SCM* argv))
+{
+  int i;
+  unsigned total_size, total_length;
+  char *q; /* first string */
+  char *p; /* others (chars or strings) */
+
+  if (argc==0) STk_error("incorrect number of arguments (%d)", argc);
+
+  /* First argument MUST be a string: */
+  if (STRINGP(argv[0]))
+    total_size = STRING_SIZE(argv[0]);
+  else
+    error_bad_string(argv[0]);
+
+  if (BOXED_INFO(argv[0]) & STRING_CONST)   error_change_const_string(argv[0]);
+
+  /* Compute total length of resulting string */
+  for (i = 1; i < argc; i++) {
+    p = argv[-i];
+    if (STRINGP(p))
+      total_size += STRING_SIZE(p);
+    else if (CHARACTERP(p))
+      total_size+=STk_utf8_char_bytes_needed(CHARACTER_VAL(p));
+    else
+      error_bad_string_or_character(p);
+  }
+
+  q = argv[0];
+
+  /* Reallocate space in first argument string: */
+  STRING_CHARS(q) = STk_must_realloc(STRING_CHARS(q), total_size);
+
+  /* point to the last character of first string: */
+  p = (STRING_CHARS(q)+STRING_SIZE(q));
+
+  STRING_SPACE(q) = STRING_SIZE(q) = total_size;
+
+  /* copy strings. i starts at ONE, not zero, since the first
+     string need not be copied. */
+  for (total_length=STRING_LENGTH(q), i=1; i < argc; i++) {
+    if (STRINGP(*(argv-i))) {
+      memcpy(p, STRING_CHARS(argv[-i]), (unsigned int) STRING_SIZE(argv[-i]));
+      p += STRING_SIZE(*(argv-i));
+      total_length += STRING_LENGTH(*(argv-i));
+    } else { /* we have already checked, it is a char! */
+      p += STk_char2utf8(CHARACTER_VAL(*(argv-i)),p);
+      /*      p += STk_utf8_char_bytes_needed(CHARACTER_VAL(p));*/
+      total_length ++;
+    }
+  }
+
+  STRING_LENGTH(q) = total_length;
+  return q;
+}
+
+inline
+static
+int get_substring_size(SCM string, long from, long to) {
+  /* WARNING: from and to must be checked by caller */
+  if (STRING_MONOBYTE(string))
+    return (to - from);
+  else {
+    /* multi-bytes string */
+    uint32_t c;
+    char *pfrom, *pto;
+
+    pto = pfrom = STk_utf8_index(STRING_CHARS(string), from, STRING_SIZE(string));
+
+    for ( ; from < to; from++)
+      pto = STk_utf8_grab_char(pto, &c);
+
+    return (pto-pfrom);
+  }
+}
+
+/*
+<doc EXT string-replace!
+ * (string-replace! dst dst-start dst-end src [src-start [src-end]])
+ *
+ * Replaces the characters of the variable-size string dst (between
+ * dst-start and dst-end) with the characters of the string src
+ * (between src-start and src-end). The number of characters from src
+ * may be different than the number replaced in dst, so the string may
+ * grow or contract. The special case where dst-start is equal to
+ * dst-end corresponds to insertion; the case where src-start is equal
+ * to src-end corresponds to deletion. The order in which characters
+ * are copied is unspecified, except that if the source and
+ * destination overlap, copying takes place as if the source is first
+ * copied into a temporary string and then into the destination.
+ * Returns string, appended with the characters form the concatenation
+ * of the given arguments, which can be wither strings or characters.
+ *
+ * It is guaranteed that string-replace! will return the same object that
+ * was passed to it as first argument, whose size may be larger.
+ * ,(linebreak)
+ * ,(bold "Note:") This function is defined in SRFI-118.
+doc>
+ */
+DEFINE_PRIMITIVE("string-replace!", string_dreplace, vsubr, (int argc, SCM* argv))
+{
+  SCM dst = argv[0];
+  SCM src = argv[-3];
+  long dst_start, dst_end, src_start, src_end;
+
+  if (!STRINGP(dst)) error_bad_string(dst);
+  if (!STRINGP(src)) error_bad_string(src);
+
+  switch (argc) {
+  case 4:  src_start = 0;
+    src_end=STRING_LENGTH(src);
+    break;
+  case 5:  if (!INTP(argv[-4])) error_bad_index(argv[-4]);
+    src_start = STk_integer_value(argv[-4]);
+    src_end=STRING_LENGTH(src);
+    if (src_start < 0 || src_start > STRING_LENGTH(src))
+      error_index_out_of_bound(src, argv[-4]);
+    break;
+  case 6:  if (!INTP(argv[-4])) error_bad_index(argv[-4]);
+    if (!INTP(argv[-5])) error_bad_index(argv[-5]);
+    src_start = STk_integer_value(argv[-4]);
+    src_end= STk_integer_value(argv[-5]);
+    if (src_start < 0 || src_start > STRING_LENGTH(src))
+      error_index_out_of_bound(src, argv[-4]);
+    if (  src_end < 0 ||   src_end > STRING_LENGTH(src))
+      error_index_out_of_bound(src, argv[-5]);
+    break;
+  default: STk_error("incorrect number of arguments (%d)", argc);
+    return STk_void;      /* for the compiler */
+  }
+
+  if (!INTP(argv[-1])) error_bad_index(argv[-1]);
+  if (!INTP(argv[-2])) error_bad_index(argv[-2]);
+  dst_start = STk_integer_value(argv[-1]);
+  dst_end   = STk_integer_value(argv[-2]);
+  if (dst_start < 0 || dst_start > STRING_LENGTH(dst))
+    error_index_out_of_bound(dst, argv[-1]);
+  if (  dst_end < 0 ||   dst_end > STRING_LENGTH(dst))
+    error_index_out_of_bound(dst, argv[-2]);
+
+  if (dst_start > dst_end)
+    STk_error("start higher than end for destination string: ~S > ~S",
+              argv[-1], argv[-2]);
+  if (src_start > src_end)
+    STk_error("start higher than end for source string: ~S > ~S",
+              argv[-4], argv[-5]);
+
+  /* if src and dest overlap, copy src to a temporary buffer and use it */
+  if (( STRING_CHARS(dst) < STRING_CHARS(src)+STRING_SIZE(src)  &&
+        STRING_CHARS(src) < STRING_CHARS(dst)+STRING_SIZE(dst) )
+      ||
+      ( STRING_CHARS(src) < STRING_CHARS(dst)+STRING_SIZE(dst) &&
+        STRING_CHARS(dst) < STRING_CHARS(dst)+STRING_SIZE(src) ))
+    src = STk_makestring(STRING_SIZE(src),STRING_CHARS(src));
+
+  int src_substring_size = get_substring_size(src, src_start, src_end);
+  int dst_substring_size = get_substring_size(dst, dst_start, dst_end);
+
+  /* how much source is larger than destination? */
+  int diff = src_substring_size - dst_substring_size;
+
+  char *start_char_dst;
+  if (diff > 0) { /* src larger, must grow dst */
+
+    /* we need to set start_char_dst here, because it will be used to move
+       elements forward and make the string larger. AFTER remalloc. */
+    STRING_CHARS(dst) = STk_must_realloc(STRING_CHARS(dst),STRING_SIZE(dst) + diff);
+    start_char_dst = STk_utf8_index(STRING_CHARS(dst),dst_start,STRING_SIZE(dst));
+    STRING_SIZE(dst) = STRING_SIZE(dst) + diff;
+
+    char *p;
+    for ( p = STRING_CHARS(dst) + STRING_SIZE(dst);
+          p >= (start_char_dst + src_substring_size);
+          p--)
+      *p = *(p-diff);
+
+  } else if (diff < 0) { /* src smaller, must shrink dst */
+
+    /* start_char_dst will change, because the string will be reallocated!
+       we compute it here, and again later */
+    start_char_dst = STk_utf8_index(STRING_CHARS(dst),dst_start,STRING_SIZE(dst));
+
+    char *p;
+    for (p = start_char_dst; p < STRING_CHARS(dst) + STRING_SIZE(dst) + diff + 1; p++)
+      *p = *(p-diff);
+
+    /* we need to set start_char_dst here, because it will be used to move
+       elements back and make the string smaller. AFTER remalloc. */
+    STRING_CHARS(dst) = STk_must_realloc(STRING_CHARS(dst), STRING_SIZE(dst) + diff);
+    start_char_dst = STk_utf8_index(STRING_CHARS(dst),dst_start,STRING_SIZE(dst));
+
+    STRING_SIZE(dst) = STRING_SIZE(dst) + diff;
+
+  } else
+    /* if the substring sizes are equal, we did not yet set the start_char_dst
+       variable. do it here. */
+    start_char_dst = STk_utf8_index(STRING_CHARS(dst),dst_start,STRING_SIZE(dst));
+
+  char* start_char_src = STk_utf8_index(STRING_CHARS(src),src_start,STRING_SIZE(src));
+  memcpy(start_char_dst, start_char_src, (unsigned long) src_substring_size);
+
+  STRING_LENGTH(dst) = STRING_LENGTH(dst) + (src_end - src_start) - (dst_end - dst_start);
+  return dst;
+}
+
+
+/*
+<doc R57RS string->list list->string
+ * (string->list string)
+ * (string->list string start)
+ * (string->list string start end)
+ * (list->string list)
+ *
+ * |String->list| returns a newly allocated list of the characters of
+ * |string| between |start| and |end|. |List->string| returns a newly
+ * allocated string formed from the characters in the list |list|,
+ * which must be a list of characters. |String->list| and
+ * |list->string| are inverses so far as |equal?| is concerned.
+ *
+ * @l
+ * ,(bold "Note"): The R5RS version of |string->list| accepts only one
+ * parameter.
+doc>
+ */
+
+/* Following version implements only the R5RS version of string->list (1 arg only) */
 DEFINE_PRIMITIVE("string->list", string2list, subr1, (SCM str))
 {
   register char *s;
@@ -723,38 +977,60 @@ DEFINE_PRIMITIVE("list->string", list2string, subr1, (SCM l))
     s += STk_char2utf8(CHARACTER_VAL(CAR(l)), s);
   }
   *s = '\0';
+
+  /* Set the length of the resulting string */
+  STRING_LENGTH(z) = len;
+
   return z;
 }
 
 
 /*
-<doc string-copy
+<doc R57RS string-copy
  * (string-copy string)
+ * (string-copy string start)
+ * (string-copy string start stop)
  *
- * Returns a newly allocated copy of the given |string|.
+ * Returns a newly allocated copy of the part of the given |string|
+ * between |start| and |stop|.
+ * @l
+ * ,(bold "Note"): The R5RS version of |string-copy| accepts only one argument.
 doc>
 */
-DEFINE_PRIMITIVE("string-copy", string_copy, subr1, (SCM str))
+DEFINE_PRIMITIVE("string-copy", string_copy, vsubr, (int argc, SCM *argv))
 {
-  if (!STRINGP(str)) error_bad_string(str);
-  return STk_makestring(STRING_SIZE(str), STRING_CHARS(str));
+  long start, end;
+
+  control_index(argc, argv, &start, &end);
+  if (start == -1)
+    return STk_makestring(STRING_SIZE(*argv), STRING_CHARS(*argv));
+  else
+    return make_substring(*argv, start, end);
 }
 
 
 /*
-<doc EXT string-fill!
+<doc R7RS string-fill!
  * (string-fill! string char)
+ * (string-fill! string char start)
+ * (string-fill! string char start end)
  *
- * Stores |char| in every element of the given |string| and returns ,(emph "void").
+ * Stores |char| in every element of the given |string| between |start| and |end|.
+ * @l
+ * ,(bold "Note"): The R5RS version of |string-fill!| accepts only one argument.
 doc>
 */
 DEFINE_PRIMITIVE("string-fill!", string_fill, subr2, (SCM str, SCM c))
 {
+  /*
+   * The following function implements the R5RS version of string-fill!
+   * The R7RS version is written in Scheme in file r7rs.stk.
+   */
   int bytes, len, c_char, c_len;
   char *s;
 
-  if (!STRINGP(str))  		      error_bad_string(str);
-  if (!CHARACTERP(c)) 		      error_bad_character(c);
+  if (!STRINGP(str))                  error_bad_string(str);
+  if (!CHARACTERP(c))                 error_bad_character(c);
   if (BOXED_INFO(str) & STRING_CONST) error_change_const_string(str);
 
   len    = STRING_LENGTH(str);
@@ -763,11 +1039,11 @@ DEFINE_PRIMITIVE("string-fill!", string_fill, subr2, (SCM str, SCM c))
   c_len  = STk_utf8_char_bytes_needed(c_char);
   bytes  = len * c_len;
 
-  if (c_len == 1) {	/* unibyte character */
+  if (c_len == 1) {     /* unibyte character */
     while (len--) {
       *s++ = c_char;
     }
-  } else {		/* multibyte character */
+  } else {              /* multibyte character */
     char buffer[5];
 
     if (bytes > STRING_SPACE(str)) {
@@ -832,25 +1108,28 @@ DEFINE_PRIMITIVE("string-find?", string_find, subr2, (SCM s1, SCM s2))
   if (!STRINGP(s2)) error_bad_string(s2);
 
   pos = Memmem(STRING_CHARS(s2), STRING_SIZE(s2),
-	       STRING_CHARS(s1), STRING_SIZE(s1),
-	       FALSE);
+               STRING_CHARS(s1), STRING_SIZE(s1),
+               FALSE);
 
   return MAKE_BOOLEAN(pos != -1);
 }
 
 /*
-<doc EXT string-index
- * (string-index str1 str2)
+<doc EXT string-index string-position
+ * (string-position str1 str2)
  *
  * Returns the (first) index where |str1| is a substring of |str2| if it exists;
  * otherwise returns |#f|.
  * @lisp
- * (string-index "ca" "abracadabra") =>  4
- * (string-index "ba" "abracadabra") =>  #f
+ * (string-position "ca" "abracadabra") =>  4
+ * (string-position "ba" "abracadabra") =>  #f
  * @end lisp
+ *
+ * ,(bold "Note") This function was also called |string-index|. This name is deprecated
+ * since it conficts with the |string-index| defined in SRFI-13.
 doc>
 */
-DEFINE_PRIMITIVE("string-index", string_index, subr2, (SCM s1, SCM s2))
+DEFINE_PRIMITIVE("string-position", string_position, subr2, (SCM s1, SCM s2))
 {
   int pos;
 
@@ -858,8 +1137,8 @@ DEFINE_PRIMITIVE("string-index", string_index, subr2, (SCM s1, SCM s2))
   if (!STRINGP(s2)) error_bad_string(s2);
 
   pos = Memmem(STRING_CHARS(s2), STRING_SIZE(s2),
-	       STRING_CHARS(s1), STRING_SIZE(s1),
-	       STk_use_utf8 && !STRING_MONOBYTE(s2));
+               STRING_CHARS(s1), STRING_SIZE(s1),
+               STk_use_utf8 && !STRING_MONOBYTE(s2));
 
   return (pos != -1) ? STk_long2integer(pos) : STk_false;
 }
@@ -903,14 +1182,14 @@ DEFINE_PRIMITIVE("string-split", string_split, subr12, (SCM string, SCM delimite
   for (s=c_string, i=0; i < l_string; s++, i++) {
     if (memchr(c_delimiters, *s, l_delimiters)) {
       if (s > c_string)
-	result = STk_cons(STk_makestring(s-c_string, c_string),
-			  result);
+        result = STk_cons(STk_makestring(s-c_string, c_string),
+                          result);
       c_string = s + 1;
     }
   }
   if (s > c_string)
     result = STk_cons(STk_makestring(s-c_string, c_string),
-		      result);
+                      result);
 
   return STk_dreverse(result);
 }
@@ -935,7 +1214,7 @@ DEFINE_PRIMITIVE("string-mutable?", string_mutable, subr1, (SCM obj))
 
 
 /*
-<doc EXT string-downcase
+<doc R7RS string-downcase
  * (string-downcase str)
  * (string-downcase str start)
  * (string-downcase str start end)
@@ -949,13 +1228,16 @@ DEFINE_PRIMITIVE("string-mutable?", string_mutable, subr1, (SCM obj))
  * (string-downcase "Foo BAR" 4)      => "bar"
  * (string-downcase "Foo BAR" 4 6)    => "ba"
  * @end lisp
+ *
+ * ,(bold "Note"): In R7RS, |string-downcase| accepts only one argument.
 doc>
  */
-static SCM string_xxcase(int argc, SCM *argv, int (*toxx)(int),
-			 wint_t (*towxx)(wint_t))
+static SCM string_xxcase(int argc, SCM *argv,
+                         int (*toxx)(int),
+                         uint32_t (*towxx)(uint32_t))
 {
   SCM s;
-  int start, end;
+  long start, end;
 
   s = control_index(argc, argv, &start, &end);
 
@@ -982,7 +1264,7 @@ static SCM string_xxcase(int argc, SCM *argv, int (*toxx)(int),
 
 DEFINE_PRIMITIVE("string-downcase", string_downcase, vsubr, (int argc, SCM *argv))
 {
-  return string_xxcase(argc, argv, tolower, towlower);
+  return string_xxcase(argc, argv, tolower, STk_to_lower);
 }
 
 
@@ -999,16 +1281,17 @@ DEFINE_PRIMITIVE("string-downcase", string_downcase, vsubr, (int argc, SCM *argv
  * @end lisp
 doc>
 */
-static SCM string_dxxcase(int argc, SCM *argv, int (*toxx)(int),
-			 wint_t (*towxx)(wint_t))
+static SCM string_dxxcase(int argc, SCM *argv,
+                          int (*toxx)(int),
+                          uint32_t (*towxx)(uint32_t))
 {
   SCM s;
-  int i, start, end;
+  long i, start, end;
 
   s    = control_index(argc, argv, &start, &end);
   if (BOXED_INFO(s) & STRING_CONST) error_change_const_string(s);
 
-  if (STk_use_utf8 && !STRING_MONOBYTE(s)) {	    /* multibyte string */
+  if (STk_use_utf8 && !STRING_MONOBYTE(s)) {        /* multibyte string */
     uint32_t *wchars;
     int len;
     char *startp = STk_utf8_index(STRING_CHARS(s), start, STRING_SIZE(s));
@@ -1021,13 +1304,13 @@ static SCM string_dxxcase(int argc, SCM *argv, int (*toxx)(int),
     }
     else {
       /* This code is inefficient, but it seems that that the converted case
-	 character always use the same length encoding. It is likely that this
-	 code is never used in practice
+         character always use the same length encoding. It is likely that this
+         code is never used in practice
       */
       for (i= start; i < end; i++)
-	STk_string_set(s, MAKE_INT(i), MAKE_CHARACTER(*wchars++));
+        STk_string_set(s, MAKE_INT(i), MAKE_CHARACTER(*wchars++));
     }
-  } else {				    /* monobyte string */
+  } else {                                  /* monobyte string */
     char *p , *endp = STRING_CHARS(s) + end;
 
     for (p=STRING_CHARS(s)+start; p < endp; p++) *p = toxx(*p);
@@ -1038,11 +1321,11 @@ static SCM string_dxxcase(int argc, SCM *argv, int (*toxx)(int),
 
 DEFINE_PRIMITIVE("string-downcase!", string_ddowncase, vsubr, (int argc, SCM *argv))
 {
-  return string_dxxcase(argc, argv, tolower, towlower);
+  return string_dxxcase(argc, argv, tolower, STk_to_lower);
 }
 
 /*
-<doc EXT string-upcase
+<doc R7RS string-upcase
  * (string-upcase str)
  * (string-upcase str start)
  * (string-upcase str start end)
@@ -1051,11 +1334,13 @@ DEFINE_PRIMITIVE("string-downcase!", string_ddowncase, vsubr, (int argc, SCM *ar
  * |start| and |end| indices have been replaced by their upper case equivalent.
  * If |start| is omited, it defaults to 0. If |end| is omited, it defaults to
  * the length of |str|.
+ * @l
+ * ,(bold "Note"): In R7RS, |string-upcase| accepts only one argument.
 doc>
  */
 DEFINE_PRIMITIVE("string-upcase", string_upcase, vsubr, (int argc, SCM *argv))
 {
-  return string_xxcase(argc, argv, toupper, towupper);
+  return string_xxcase(argc, argv, toupper, STk_to_upper);
 }
 
 /*
@@ -1069,12 +1354,12 @@ doc>
 */
 DEFINE_PRIMITIVE("string-upcase!", string_dupcase, vsubr, (int argc, SCM *argv))
 {
-  return string_dxxcase(argc, argv, toupper, towupper);
+  return string_dxxcase(argc, argv, toupper, STk_to_upper);
 }
 
 
 /*
-<doc EXT string-foldcase
+<doc R7RS string-foldcase
  * (string-foldcase str)
  * (string-foldcase str start)
  * (string-foldcase str start end)
@@ -1083,11 +1368,13 @@ DEFINE_PRIMITIVE("string-upcase!", string_dupcase, vsubr, (int argc, SCM *argv))
  * been applied on |str| between the |start| and |end| indices.
  * If |start| is omited, it defaults to 0. If |end| is omited, it defaults to
  * the length of |str|.
+ * @l
+ * ,(bold "Note"): In R7RS, |string-foldcase| accepts only one argument.
 doc>
  */
 DEFINE_PRIMITIVE("string-foldcase", string_foldcase, vsubr, (int argc, SCM *argv))
 {
-  return string_xxcase(argc, argv, tolower, (wint_t (*) (wint_t))STk_casefold_char);
+  return string_xxcase(argc, argv, tolower, STk_to_fold);
 }
 
 /*
@@ -1101,7 +1388,8 @@ doc>
 */
 DEFINE_PRIMITIVE("string-foldcase!", string_dfoldcase, vsubr, (int argc, SCM *argv))
 {
-  return string_dxxcase(argc, argv, toupper,(wint_t (*) (wint_t))STk_casefold_char);
+  return string_dxxcase(argc, argv, toupper, STk_to_fold);
+
 }
 
 
@@ -1133,7 +1421,7 @@ doc>
 DEFINE_PRIMITIVE("string-titlecase", string_titlecase, vsubr, (int argc, SCM *argv))
 {
   SCM s, z;
-  int start, end;
+  long start, end;
   char *endp, *p, *q;
   char prev_is_sep = 1, curr_is_sep;
 
@@ -1164,7 +1452,7 @@ doc>
 DEFINE_PRIMITIVE("string-titlecase!", string_dtitlecase,vsubr,(int argc, SCM *argv))
 {
   SCM s;
-  int start, end;
+  long start, end;
   char *endp, *p;
   char prev_is_sep = 1, curr_is_sep;
 
@@ -1202,7 +1490,7 @@ DEFINE_PRIMITIVE("string-titlecase!", string_dtitlecase,vsubr,(int argc, SCM *ar
 doc>
 */
 DEFINE_PRIMITIVE("string-blit!", string_blit, subr3,
-		 (SCM str1, SCM str2, SCM offset))
+                 (SCM str1, SCM str2, SCM offset))
 {
   long off = STk_integer_value(offset);
   int len1, len2;
@@ -1235,11 +1523,11 @@ DEFINE_PRIMITIVE("string-blit!", string_blit, subr3,
 
     for (i = 0; i < newl; i++) {
       if ((i >= off) && (i < (off + len2)))
-	*snew++ = sstr2[j++];
+        *snew++ = sstr2[j++];
       else if (i < len1)
-	*snew++ =  sstr1[i];
+        *snew++ =  sstr1[i];
       else
-	snew++;
+        snew++;
     }
     return new;
   }
@@ -1252,7 +1540,7 @@ DEFINE_PRIMITIVE("string-pos", string_pos, subr2, (SCM str, SCM index))
   long k = STk_integer_value(index);
   int n;
 
-  if (!STRINGP(str))			error_bad_string(str);
+  if (!STRINGP(str))                    error_bad_string(str);
   if (k < 0 || k >= STRING_LENGTH(str)) error_index_out_of_bound(str, index);
 
   if (STk_use_utf8 && !STRING_MONOBYTE(str))
@@ -1263,6 +1551,22 @@ DEFINE_PRIMITIVE("string-pos", string_pos, subr2, (SCM str, SCM index))
 */
 
 
+DEFINE_PRIMITIVE("string->utf8", string2utf8, vsubr, (int argc, SCM *argv))
+{
+  long start, end;
+  SCM str;
+  char *start_addr, *end_addr;
+
+
+  str           = control_index(argc, argv, &start, &end);
+  start_addr    = STk_utf8_index(STRING_CHARS(str), (int) start, STRING_SIZE(str));
+  end_addr      = STk_utf8_index(STRING_CHARS(str), (int) end, STRING_SIZE(str));
+
+  return STk_make_bytevector_from_C_string(start_addr, end_addr - start_addr);
+}
+
+
+
 DEFINE_PRIMITIVE("%use-utf8?", using_utf8, subr0, (void))
 {
   return MAKE_BOOLEAN(STk_use_utf8);
@@ -1312,13 +1616,15 @@ int STk_init_string(void)
   ADD_PRIMITIVE(strgei);
   ADD_PRIMITIVE(substring);
   ADD_PRIMITIVE(string_append);
+  ADD_PRIMITIVE(string_dappend);
+  ADD_PRIMITIVE(string_dreplace);
   ADD_PRIMITIVE(string2list);
   ADD_PRIMITIVE(list2string);
   ADD_PRIMITIVE(string_copy);
   ADD_PRIMITIVE(string_fill);
 
   ADD_PRIMITIVE(string_find);
-  ADD_PRIMITIVE(string_index);
+  ADD_PRIMITIVE(string_position);
   ADD_PRIMITIVE(string_split);
   ADD_PRIMITIVE(string_mutable);
   ADD_PRIMITIVE(string_downcase);
@@ -1331,6 +1637,7 @@ int STk_init_string(void)
   ADD_PRIMITIVE(string_dtitlecase);
   ADD_PRIMITIVE(string_blit);
 
+  ADD_PRIMITIVE(string2utf8);
   ADD_PRIMITIVE(using_utf8);
   ADD_PRIMITIVE(string_use_utf8);
   ADD_PRIMITIVE(string2bytes);
