@@ -1,31 +1,39 @@
 /*
  *
- * s y s t e m . c				-- System relative primitives
+ * s y s t e m . c                              -- System relative primitives
  *
- * Copyright © 1994-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 1994-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
- * Permission to use, copy, modify, distribute,and license this
- * software and its documentation for any purpose is hereby granted,
- * provided that existing copyright notices are retained in all
- * copies and that this notice is included verbatim in any
- * distributions.  No written agreement, license, or royalty fee is
- * required for any of the authorized uses.
- * This software is provided ``AS IS'' without express or implied
- * warranty.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ * USA.
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 29-Mar-1994 10:57
- * Last file update: 27-May-2011 22:47 (eg)
+ * Last file update: 10-Sep-2021 11:37 (eg)
  */
 
 #include <unistd.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/utsname.h>
 #include <fcntl.h>
 #include <dirent.h>
 #include <time.h>
+#include <locale.h>
 #include "stklos.h"
 #include "struct.h"
 
@@ -42,6 +50,7 @@
 
 static SCM exit_procs = STk_nil; /* atexit functions */
 static SCM date_type, time_type;
+static SCM *temp_file_prefix;
 
 /******************************************************************************
  *
@@ -58,6 +67,11 @@ static void error_bad_string(SCM path)
   STk_error("~S is a bad string", path);
 }
 
+static void error_bad_int(SCM n)
+{
+  STk_error("~S is a bad int", n);
+}
+
 static void error_bad_int_or_out_of_bounds(SCM val)
 {
   STk_error("bad integer ~S (or out of range)", val);
@@ -170,6 +184,488 @@ int STk_dirp(const char *path)
 //EG: }
 #endif
 
+/******************************************************************************
+ *
+ *  SRFI 170 support
+ *
+ ******************************************************************************/
+#define CASE_ERRNO(x)    case x: err = #x; break;
+
+static SCM get_posix_error_name (int n)
+{
+  char *err = "";
+
+  switch (n) {
+#ifdef E2BIG
+    CASE_ERRNO(E2BIG)
+#endif
+#ifdef EACCES
+    CASE_ERRNO(EACCES)
+#endif
+#ifdef EADDRINUSE
+    CASE_ERRNO(EADDRINUSE)
+#endif
+#ifdef EADDRNOTAVAIL
+    CASE_ERRNO(EADDRNOTAVAIL)
+#endif
+#ifdef EAFNOSUPPORT
+    CASE_ERRNO(EAFNOSUPPORT)
+#endif
+#ifdef EAGAIN
+    CASE_ERRNO(EAGAIN)
+#endif
+#ifdef EALREADY
+    CASE_ERRNO(EALREADY)
+#endif
+#ifdef EBADE
+    CASE_ERRNO(EBADE)
+#endif
+#ifdef EBADF
+    CASE_ERRNO(EBADF)
+#endif
+#ifdef EBADFD
+    CASE_ERRNO(EBADFD)
+#endif
+#ifdef EBADMSG
+    CASE_ERRNO(EBADMSG)
+#endif
+#ifdef EBADR
+    CASE_ERRNO(EBADR)
+#endif
+#ifdef EBADRQC
+    CASE_ERRNO(EBADRQC)
+#endif
+#ifdef EBADSLT
+    CASE_ERRNO(EBADSLT)
+#endif
+#ifdef EBUSY
+    CASE_ERRNO(EBUSY)
+#endif
+#ifdef ECANCELED
+    CASE_ERRNO(ECANCELED)
+#endif
+#ifdef ECHILD
+    CASE_ERRNO(ECHILD)
+#endif
+#ifdef ECHRNG
+    CASE_ERRNO(ECHRNG)
+#endif
+#ifdef ECOMM
+    CASE_ERRNO(ECOMM)
+#endif
+#ifdef ECONNABORTED
+    CASE_ERRNO(ECONNABORTED)
+#endif
+#ifdef ECONNREFUSED
+    CASE_ERRNO(ECONNREFUSED)
+#endif
+#ifdef ECONNRESET
+    CASE_ERRNO(ECONNRESET)
+#endif
+#ifdef EDEADLK
+    CASE_ERRNO(EDEADLK)
+#endif
+#ifdef EDESTADDRREQ
+    CASE_ERRNO(EDESTADDRREQ)
+#endif
+#ifdef EDOM
+    CASE_ERRNO(EDOM)
+#endif
+#ifdef EDQUOT
+    CASE_ERRNO(EDQUOT)
+#endif
+#ifdef EEXIST
+    CASE_ERRNO(EEXIST)
+#endif
+#ifdef EFAULT
+    CASE_ERRNO(EFAULT)
+#endif
+#ifdef EFBIG
+    CASE_ERRNO(EFBIG)
+#endif
+#ifdef EHOSTDOWN
+    CASE_ERRNO(EHOSTDOWN)
+#endif
+#ifdef EHOSTUNREACH
+    CASE_ERRNO(EHOSTUNREACH)
+#endif
+#ifdef EHWPOISON
+    CASE_ERRNO(EHWPOISON)
+#endif
+#ifdef EIDRM
+    CASE_ERRNO(EIDRM)
+#endif
+#ifdef EILSEQ
+    CASE_ERRNO(EILSEQ)
+#endif
+#ifdef EINPROGRESS
+    CASE_ERRNO(EINPROGRESS)
+#endif
+#ifdef EINTR
+    CASE_ERRNO(EINTR)
+#endif
+#ifdef EINVAL
+    CASE_ERRNO(EINVAL)
+#endif
+#ifdef EIO
+    CASE_ERRNO(EIO)
+#endif
+#ifdef EISCONN
+    CASE_ERRNO(EISCONN)
+#endif
+#ifdef EISDIR
+    CASE_ERRNO(EISDIR)
+#endif
+#ifdef EISNAM
+    CASE_ERRNO(EISNAM)
+#endif
+#ifdef EKEYEXPIRED
+    CASE_ERRNO(EKEYEXPIRED)
+#endif
+#ifdef EKEYREJECTED
+    CASE_ERRNO(EKEYREJECTED)
+#endif
+#ifdef EKEYREVOKED
+    CASE_ERRNO(EKEYREVOKED)
+#endif
+#ifdef EL2HLT
+    CASE_ERRNO(EL2HLT)
+#endif
+#ifdef EL2NSYNC
+    CASE_ERRNO(EL2NSYNC)
+#endif
+#ifdef EL3HLT
+    CASE_ERRNO(EL3HLT)
+#endif
+#ifdef EL3RST
+    CASE_ERRNO(EL3RST)
+#endif
+#ifdef ELIBACC
+    CASE_ERRNO(ELIBACC)
+#endif
+#ifdef ELIBBAD
+    CASE_ERRNO(ELIBBAD)
+#endif
+#ifdef ELIBMAX
+    CASE_ERRNO(ELIBMAX)
+#endif
+#ifdef ELIBSCN
+    CASE_ERRNO(ELIBSCN)
+#endif
+#ifdef ELIBEXEC
+    CASE_ERRNO(ELIBEXEC)
+#endif
+#ifdef ELNRANGE
+    CASE_ERRNO(ELNRANGE)
+#endif
+#ifdef ELOOP
+    CASE_ERRNO(ELOOP)
+#endif
+#ifdef EMEDIUMTYPE
+    CASE_ERRNO(EMEDIUMTYPE)
+#endif
+#ifdef EMFILE
+    CASE_ERRNO(EMFILE)
+#endif
+#ifdef EMLINK
+    CASE_ERRNO(EMLINK)
+#endif
+#ifdef EMSGSIZE
+    CASE_ERRNO(EMSGSIZE)
+#endif
+#ifdef EMULTIHOP
+    CASE_ERRNO(EMULTIHOP)
+#endif
+#ifdef ENAMETOOLONG
+    CASE_ERRNO(ENAMETOOLONG)
+#endif
+#ifdef ENETDOWN
+    CASE_ERRNO(ENETDOWN)
+#endif
+#ifdef ENETRESET
+    CASE_ERRNO(ENETRESET)
+#endif
+#ifdef ENETUNREACH
+    CASE_ERRNO(ENETUNREACH)
+#endif
+#ifdef ENFILE
+    CASE_ERRNO(ENFILE)
+#endif
+#ifdef ENOANO
+    CASE_ERRNO(ENOANO)
+#endif
+#ifdef ENOBUFS
+    CASE_ERRNO(ENOBUFS)
+#endif
+#ifdef ENODATA
+    CASE_ERRNO(ENODATA)
+#endif
+#ifdef ENODEV
+    CASE_ERRNO(ENODEV)
+#endif
+#ifdef ENOENT
+    CASE_ERRNO(ENOENT)
+#endif
+#ifdef ENOEXEC
+    CASE_ERRNO(ENOEXEC)
+#endif
+#ifdef ENOKEY
+    CASE_ERRNO(ENOKEY)
+#endif
+#ifdef ENOLCK
+    CASE_ERRNO(ENOLCK)
+#endif
+#ifdef ENOLINK
+    CASE_ERRNO(ENOLINK)
+#endif
+#ifdef ENOMEDIUM
+    CASE_ERRNO(ENOMEDIUM)
+#endif
+#ifdef ENOMEM
+    CASE_ERRNO(ENOMEM)
+#endif
+#ifdef ENOMSG
+    CASE_ERRNO(ENOMSG)
+#endif
+#ifdef ENONET
+    CASE_ERRNO(ENONET)
+#endif
+#ifdef ENOPKG
+    CASE_ERRNO(ENOPKG)
+#endif
+#ifdef ENOPROTOOPT
+    CASE_ERRNO(ENOPROTOOPT)
+#endif
+#ifdef ENOSPC
+    CASE_ERRNO(ENOSPC)
+#endif
+#ifdef ENOSR
+    CASE_ERRNO(ENOSR)
+#endif
+#ifdef ENOSTR
+    CASE_ERRNO(ENOSTR)
+#endif
+#ifdef ENOSYS
+    CASE_ERRNO(ENOSYS)
+#endif
+#ifdef ENOTBLK
+    CASE_ERRNO(ENOTBLK)
+#endif
+#ifdef ENOTCONN
+    CASE_ERRNO(ENOTCONN)
+#endif
+#ifdef ENOTDIR
+    CASE_ERRNO(ENOTDIR)
+#endif
+#ifdef ENOTEMPTY
+    CASE_ERRNO(ENOTEMPTY)
+#endif
+#ifdef ENOTRECOVERABLE
+    CASE_ERRNO(ENOTRECOVERABLE)
+#endif
+#ifdef ENOTSOCK
+    CASE_ERRNO(ENOTSOCK)
+#endif
+#ifdef ENOTSUP
+    CASE_ERRNO(ENOTSUP)
+#endif
+#ifdef ENOTTY
+    CASE_ERRNO(ENOTTY)
+#endif
+#ifdef ENOTUNIQ
+    CASE_ERRNO(ENOTUNIQ)
+#endif
+#ifdef ENXIO
+    CASE_ERRNO(ENXIO)
+#endif
+#ifdef EOVERFLOW
+    CASE_ERRNO(EOVERFLOW)
+#endif
+#ifdef EOWNERDEAD
+    CASE_ERRNO(EOWNERDEAD)
+#endif
+#ifdef EPERM
+    CASE_ERRNO(EPERM)
+#endif
+#ifdef EPFNOSUPPORT
+    CASE_ERRNO(EPFNOSUPPORT)
+#endif
+#ifdef EPIPE
+    CASE_ERRNO(EPIPE)
+#endif
+#ifdef EPROTO
+    CASE_ERRNO(EPROTO)
+#endif
+#ifdef EPROTONOSUPPORT
+    CASE_ERRNO(EPROTONOSUPPORT)
+#endif
+#ifdef EPROTOTYPE
+    CASE_ERRNO(EPROTOTYPE)
+#endif
+#ifdef ERANGE
+    CASE_ERRNO(ERANGE)
+#endif
+#ifdef EREMCHG
+    CASE_ERRNO(EREMCHG)
+#endif
+#ifdef EREMOTE
+    CASE_ERRNO(EREMOTE)
+#endif
+#ifdef EREMOTEIO
+    CASE_ERRNO(EREMOTEIO)
+#endif
+#ifdef ERESTART
+    CASE_ERRNO(ERESTART)
+#endif
+#ifdef ERFKILL
+    CASE_ERRNO(ERFKILL)
+#endif
+#ifdef EROFS
+    CASE_ERRNO(EROFS)
+#endif
+#ifdef ESHUTDOWN
+    CASE_ERRNO(ESHUTDOWN)
+#endif
+#ifdef ESPIPE
+    CASE_ERRNO(ESPIPE)
+#endif
+#ifdef ESOCKTNOSUPPORT
+    CASE_ERRNO(ESOCKTNOSUPPORT)
+#endif
+#ifdef ESRCH
+    CASE_ERRNO(ESRCH)
+#endif
+#ifdef ESTALE
+    CASE_ERRNO(ESTALE)
+#endif
+#ifdef ESTRPIPE
+    CASE_ERRNO(ESTRPIPE)
+#endif
+#ifdef ETIME
+    CASE_ERRNO(ETIME)
+#endif
+#ifdef ETIMEDOUT
+    CASE_ERRNO(ETIMEDOUT)
+#endif
+#ifdef ETOOMANYREFS
+    CASE_ERRNO(ETOOMANYREFS)
+#endif
+#ifdef ETXTBSY
+    CASE_ERRNO(ETXTBSY)
+#endif
+#ifdef EUCLEAN
+    CASE_ERRNO(EUCLEAN)
+#endif
+#ifdef EUNATCH
+    CASE_ERRNO(EUNATCH)
+#endif
+#ifdef EUSERS
+    CASE_ERRNO(EUSERS)
+#endif
+#ifdef EXDEV
+    CASE_ERRNO(EXDEV)
+#endif
+#ifdef EXFULL
+    CASE_ERRNO(EXFULL)
+#endif
+#ifdef EOPNOTSUPP
+  #ifdef ENOTSUP
+     #if ENOTSUP != EOPNOTSUPP
+       CASE_ERRNO(EOPNOTSUPP)
+     #endif
+  #else
+      CASE_ERRNO(EOPNOTSUPP)
+  #endif
+#endif
+#ifdef EWOULDBLOCK
+  #ifdef EAGAIN
+     #if EAGAIN != EWOULDBLOCK
+       CASE_ERRNO(EWOULDBLOCK)
+     #endif
+  #else
+      CASE_ERRNO(EWOULDBLOCK)
+  #endif
+#endif
+#ifdef EDEADLOCK
+  #ifdef EDEADLK
+     #if EDEADLK != EDEADLOCK
+       CASE_ERRNO(EDEADLOCK)
+     #endif
+  #else
+      CASE_ERRNO(EDEADLOCK)
+  #endif
+#endif
+    default:            err = "UNKNOWN_NAME";break;
+  }
+  return STk_intern(err);
+}
+
+
+/* Raise a &posix-error message */
+void STk_error_posix(int err, char *proc_name, SCM obj1, SCM obj2)
+{
+    SCM err_no = MAKE_INT(err);
+    SCM errname = get_posix_error_name(err);
+    SCM r7_msg = STk_Cstring2string(strerror(err));
+    SCM procedure = (*proc_name) ? STk_intern(proc_name): STk_false;
+    SCM args, message;
+
+
+    /* Build the error message */
+    if (obj1 && obj2) {
+      message = STk_format_error("~a: ~s, ~s", r7_msg, obj1, obj2);
+      args    = LIST2(obj1, obj2);
+    } else if (obj1) {
+      message = STk_format_error("~a: ~s", r7_msg, obj1);
+      args    = LIST1(obj1);
+    } else {
+      message = r7_msg;
+      args    = STk_nil;
+    }
+
+    STk_raise_exception(STk_make_C_cond(STk_posix_error_condition,
+                                        7,
+                                        procedure,    /* location  */
+                                        STk_vm_bt(),  /* backtrace */
+                                        message,      /* message */
+                                        r7_msg,       /* r7rs-msg */
+                                        args,         /* r7rs-irritants */
+                                        errname,      /* errname */
+                                        err_no));     /* errno */
+}
+
+
+/*
+<doc EXT temp-file-prefix
+ * (temp-file-prefix)
+ * (temp-file-prefix value)
+ *
+ * This parameter object permits to change the default prefix used to build
+ * temporary file name. Its default value is built using the |TMPDIR|
+ * environment variable (if it is defined) and the current process ID.
+ * If a value is provided, it must be a string designating a valid prefix path.
+ * @l
+ * This parameter object is also defined in ,(link-srfi 170).
+doc>
+*/
+static SCM init_temp_file_prefix(void)
+{
+  char *tmpdir = getenv("TMPDIR");
+  char buffer[MAX_PATH_LENGTH];
+
+  if (!tmpdir) tmpdir = "/tmp";
+  snprintf(buffer, MAX_PATH_LENGTH, "%s/%d", tmpdir, getpid());
+
+  return temp_file_prefix = STk_Cstring2string(buffer);
+}
+
+static SCM temp_file_prefix_conv(SCM val)
+{
+  if (!STRINGP(val)) error_bad_string(val);
+  return (temp_file_prefix = val);
+}
+
+
 /******************************************************************************
  *
  * Primitives
@@ -193,7 +689,7 @@ int STk_dirp(const char *path)
  * @end lisp
 doc>
 */
-DEFINE_PRIMITIVE("winify-file-name", winify_filename, subr1, (SCM f))
+DEFINE_PRIMITIVE("winify-file-name", winify_filename, subr1, (SCM _UNUSED(f)))
 {
 #ifdef WIN32
   char expanded[2 * MAX_PATH_LENGTH];
@@ -224,7 +720,7 @@ DEFINE_PRIMITIVE("winify-file-name", winify_filename, subr1, (SCM f))
  * @end lisp
 doc>
 */
-DEFINE_PRIMITIVE("posixify-file-name", posixify_filename, subr1, (SCM f))
+DEFINE_PRIMITIVE("posixify-file-name", posixify_filename, subr1, (SCM _UNUSED(f)))
 {
 #ifdef WIN32
   char expanded[2 * MAX_PATH_LENGTH];
@@ -317,39 +813,58 @@ DEFINE_PRIMITIVE("chdir", chdir, subr1, (SCM s))
 }
 
 /*
-<doc EXT make-directory
- * (make-directory dir)
+<doc EXT make-directory create-directory
+ * (create-directory dir)
+ * (create-directory dir permissions)
  *
- * Create a directory with name |dir|.
+ * Create a directory with name |dir|. If |permissions| is omitted, it
+ * defaults to #o775 (masked by the current umask).
+ * ,(linebreak)
+ * This function is also defined in ,(link-srfi 170). The old name
+ * |make-directory| is deprecated.
 doc>
 */
-DEFINE_PRIMITIVE("make-directory", make_directory, subr1, (SCM path))
+DEFINE_PRIMITIVE("create-directory", make_directory, subr12, (SCM path, SCM perms))
 {
-  mode_t mask;
+  mode_t msk;
 
   if (!STRINGP(path)) error_bad_path(path);
-  umask(mask = umask(0));
-  if (mkdir(STRING_CHARS(path), 0777) != 0)
-    error_posix(path, NULL);
+
+  if (perms) {
+    if (!INTP(perms)) error_bad_int(perms);
+    msk = INT_VAL(perms);
+  }
+  else {
+    umask(msk = umask(0));
+    msk = 0775;
+  }
+
+  if (mkdir(STRING_CHARS(path), msk) != 0)
+    STk_error_posix(errno, "", path, perms);
+
   return STk_void;
 }
 
+
 /*
-<doc EXT remove-directory
+<doc EXT delete-directory remove-directory
+ * (delete-directory dir)
  * (remove-directory dir)
  *
  * Delete the directory with name |dir|.
+ * @l
+ * ,(bold "Note:") This function is also defined in ,(link-srfi 170). The name
+ * |remove-directory| is kept for compatibility.
 doc>
 */
-DEFINE_PRIMITIVE("remove-directory", remove_directory, subr1, (SCM path))
+DEFINE_PRIMITIVE("delete-directory", delete_directory, subr1, (SCM path))
 {
   if (!STRINGP(path)) error_bad_path(path);
   if (rmdir(STRING_CHARS(path)) != 0)
-    error_posix(path, NULL);
+    STk_error_posix(errno, "", path, NULL);
   return STk_void;
 }
 
-
 /*
 <doc EXT getpid
  * (getpid)
@@ -379,13 +894,12 @@ DEFINE_PRIMITIVE("system", system, subr1, (SCM com))
 }
 
 /*
-<doc EXT file-is-directory? file-is-regular? file-is-writable? file-is-readable? file-is-executable? file-exists?
+<doc EXT file-is-directory? file-is-regular? file-is-writable? file-is-readable? file-is-executable?
  * (file-is-directory?  string)
  * (file-is-regular?    string)
  * (file-is-readable?   string)
  * (file-is-writable?   string)
  * (file-is-executable? string)
- * (file-exists?        string)
  *
  * Returns |#t| if the predicate is true for the path name given in
  * |string|; returns |#f| otherwise (or if |string| denotes a file
@@ -428,6 +942,15 @@ DEFINE_PRIMITIVE("file-is-executable?", file_is_executablep, subr1, (SCM f))
 }
 
 
+
+/*
+<doc R7RS file-exists?
+ * (file-exists? string)
+ *
+ * Returns |#t| if the path name given in |string| denotes an existing file;
+ * returns |#f| otherwise.
+doc>
+ */
 DEFINE_PRIMITIVE("file-exists?", file_existsp, subr1, (SCM f))
 {
   struct stat info;
@@ -495,19 +1018,30 @@ DEFINE_PRIMITIVE("glob", glob, vsubr, (int argc, SCM *argv))
 
 
 /*
-<doc EXT remove-file
- * (remove-file string)
+<doc R7RS delete-file remove-file
+ * (delete-file string)
  *
  * Removes the file whose path name is given in |string|.
- * The result of |remove-file| is ,(emph "void").
+ * The result of |delete-file| is ,(emph "void").
+ * @l
+ * This function is also called |remove-file| for compatibility
+ * reasons. ,(index "remove-file")
 doc>
 */
+#define do_remove(filename)                             \
+  if (!STRINGP(filename)) error_bad_string(filename);   \
+  if (remove(STRING_CHARS(filename)) != 0)              \
+    STk_error_posix(errno, "", filename, NULL);       \
+  return STk_void;                                      \
+
+DEFINE_PRIMITIVE("delete-file", delete_file, subr1, (SCM filename))
+{
+  do_remove(filename);
+}
+
 DEFINE_PRIMITIVE("remove-file", remove_file, subr1, (SCM filename))
 {
-  if (!STRINGP(filename)) error_bad_string(filename);
-  if (remove(STRING_CHARS(filename)) != 0)
-    error_posix(filename, NULL);
-  return STk_void;
+  do_remove(filename);
 }
 
 
@@ -517,6 +1051,8 @@ DEFINE_PRIMITIVE("remove-file", remove_file, subr1, (SCM filename))
  *
  * Renames the file whose path-name is |string1| to a file whose path-name is
  * |string2|. The result of |rename-file| is ,(emph "void").
+ * @l
+ * This function is also defined in ,(link-srfi 170).
 doc>
 */
 DEFINE_PRIMITIVE("rename-file", rename_file, subr2, (SCM filename1, SCM filename2))
@@ -524,7 +1060,7 @@ DEFINE_PRIMITIVE("rename-file", rename_file, subr2, (SCM filename1, SCM filename
   if (!STRINGP(filename1)) error_bad_string(filename1);
   if (!STRINGP(filename2)) error_bad_string(filename2);
   if (rename(STRING_CHARS(filename1), STRING_CHARS(filename2)) != 0)
-    error_posix(filename1, filename2);
+    STk_error_posix(errno, "", filename1, filename2);
   return STk_void;
 }
 
@@ -532,27 +1068,38 @@ DEFINE_PRIMITIVE("rename-file", rename_file, subr2, (SCM filename1, SCM filename
 /*
 <doc EXT directory-files
  * (directory-files path)
+ * (directory-files path dotfiles?)
  *
- * Returns the list of the files in the directory |path|. Directories ,(q ".")
- * and ,(q "..") don't appear in the result.
+ * Returns the list of the files in the directory |path|. The |dotfiles?| flag
+ * (default |#f|) causes files beginning with ,(q ".") to be included in the list.
+ * Regardless of the value of |dotfiles?|, the two files ,(q ".") and ,(q "..")
+ * are never  returned.
+ * @l
+ * This function is also defined in ,(link-srfi 170).
 doc>
 */
-DEFINE_PRIMITIVE("directory-files", directory_files, subr1, (SCM dirname))
+DEFINE_PRIMITIVE("directory-files", directory_files, subr12, (SCM dirname, SCM dot))
 {
   char *path;
   DIR* dir;
   SCM res = STk_nil;
   struct dirent *d;
+  int add_dot_files = dot? (dot  != STk_false): 0;
 
   if (!STRINGP(dirname)) error_bad_string(dirname);
   path = STk_expand_file_name(STRING_CHARS(dirname));
   dir  = opendir(path);
-  if (!dir) error_posix(dirname, NULL);
+  if (!dir) STk_error_posix(errno, "", dirname, NULL);
 
+  /* readdir and closedir can yield an error (EBADF) only on  when dir is incorrect
+   * This cannot occurs here since we have tested that opendir result is OK.
+   */
   for (d = readdir(dir); d ; d = readdir(dir)) {
-    if (d->d_name[0] == '.')
-      if ((d->d_name[1] == '\0') || (d->d_name[1] == '.' && d->d_name[2] == '\0'))
-	continue;
+    if (d->d_name[0] == '.') {
+      if (!add_dot_files ||
+          ((d->d_name[1] == '\0') || (d->d_name[1] == '.' && d->d_name[2] == '\0')))
+        continue;
+    }
     res = STk_cons(STk_Cstring2string(d->d_name), res);
   }
   closedir(dir);
@@ -597,37 +1144,88 @@ DEFINE_PRIMITIVE("copy-file", copy_file, subr2, (SCM filename1, SCM filename2))
 
 
 /*
-<doc EXT temporary-file-name
- * (temporary-file-name)
+<doc EXT temporary-file-name create-temp-file
+ * (create-temp-file)
+ * (create-temp-file prefix)
  *
- * Generates a unique temporary file name. The value returned by
- * |temporary-file-name| is the newly generated name of |#f|
- * if a unique name cannot be generated.
+ * Creates a new temporary file and returns two values: its name and an opened
+ * file port on it. The optional argument specifies the filename prefix
+ * to use, and defaults to the result of invoking |temp-file-prefix|.
+ * The returned file port is opened in read/write mode. It ensures that
+ * the name cannot be reused by another process before being used
+ * in the program that calls |create-temp-file|.
+ * Note, that if the opened port is not used, it can be closed and collected
+ * by the GC.
+ * @lisp
+ * (let-values (((name port) (create-temp-file)))
+ *   (let ((msg (format "Name: ~s\n" name)))
+ *     (display msg)
+ *     (display msg port)
+ *     (close-port port)))     => prints the name of the temp. file on the
+ *                                current output port and in the file itself.
+ * @end lisp
+ *
+ * ,(bold "Notes:") This function is also defined in ,(link-srfi 170).However,
+ * in SRFI-170, |create-temp-file| returns only the name of the temporary file.
+ * @l
+ * |temporary-file-name| is another name for this function.
 doc>
 */
-DEFINE_PRIMITIVE("temporary-file-name", tmp_file, subr0, (void))
+
+static void initialize_temp_template(char *buffer, SCM arg)
 {
-#ifdef WIN32
-  char buff[MAX_PATH_LENGTH], *s;
+  SCM prefix = temp_file_prefix;
 
-  s = tmpnam(buff);
-  return s ? STk_Cstring2string(s) : STk_false;
-#else
-  static int cpt=0;
-  char buff[MAX_PATH_LENGTH];
-  MUT_DECL(tmpnam_mutex);
-
-  MUT_LOCK(tmpnam_mutex);
-  for ( ; ; ) {
-    sprintf(buff, "/tmp/stklos%05x", cpt++);
-    if (cpt > 100000)		/* arbitrary limit to avoid infinite search */
-      return STk_false;
-    if (access(buff, F_OK) == -1) break;
+  if (arg){
+    if (!STRINGP(arg)) error_bad_string(arg);
+    prefix = arg;
   }
-  MUT_UNLOCK(tmpnam_mutex);
+  snprintf(buffer, MAX_PATH_LENGTH, "%sXXXXXX", STRING_CHARS(prefix));
+}
 
-  return STk_Cstring2string(buff);
-#endif
+
+DEFINE_PRIMITIVE("create-temp-file", create_tmp_file, subr01, (SCM arg))
+{
+  char buffer[MAX_PATH_LENGTH];
+  SCM port, res[2];
+  int fd;
+
+  initialize_temp_template(buffer, arg);
+  fd = mkstemp(buffer);
+  if (fd<0) STk_error_posix(errno,"", arg, NULL);
+
+  /* Since fd designs an open port, create a Scheme port to avoid a file descriptor
+   * leak (it will be GC'ed if not used). The result will be two values (the name
+   * of the temporary file name and a write port associated to the file
+   */
+  port = STk_fd2scheme_port(fd, "w", buffer);
+  PORT_FLAGS(port) |= PORT_TEXTUAL | PORT_BINARY;
+
+  res[0]= port;
+  res[1]= STk_Cstring2string(buffer);
+  return STk_values(2, res +1);
+}
+
+/*
+<doc EXT create-temp-directory
+ * (create-temp-directory)
+ * (create-temp-directory prefix)
+ *
+ * Creates a new temporary directory and returns its name as a string.
+ * The optional argument specifies the filename prefix to use, and
+ * defaults to the result of invoking |temp-file-prefix|.
+doc>
+*/
+DEFINE_PRIMITIVE("create-temp-directory", create_tmp_dir, subr01, (SCM arg))
+{
+  char buffer[MAX_PATH_LENGTH];
+  char *dir;
+
+  initialize_temp_template(buffer, arg);
+  if ((dir = mkdtemp(buffer)) == NULL) {
+    STk_error_posix(errno,"", arg, NULL);
+  }
+  return STk_Cstring2string(dir);
 }
 
 
@@ -645,12 +1243,12 @@ DEFINE_PRIMITIVE("temporary-file-name", tmp_file, subr0, (void))
  *        (out (open-output-file tmp)))
  *   (register-exit-function! (lambda (n)
  *                              (when (zero? n)
- *                                (remove-file tmp))))
+ *                                (delete-file tmp))))
  *   out)
  * @end lisp
 doc>
 */
-MUT_DECL(at_exit_mutex);	 /* The exit mutex */
+MUT_DECL(at_exit_mutex)         /* The exit mutex */
 
 DEFINE_PRIMITIVE("register-exit-function!", at_exit, subr1, (SCM proc))
 {
@@ -686,21 +1284,58 @@ DEFINE_PRIMITIVE("%pre-exit", pre_exit, subr1, (SCM retcode))
  * is omitted, the program terminates with a return code of 0.
  * If  program has registered exit functions with |register-exit-function!|,
  * they are called (in an order which is the reverse of their call order).
+ * @l
+ * ,(bold "Note:") The ,(stklos) |exit| primitive accepts also an
+ * integer value as parameter (,(rseven) accepts only a boolean).
 doc>
 */
 DEFINE_PRIMITIVE("exit", exit, subr01, (SCM retcode))
 {
   long ret = 0;
+  SCM cond;
 
   if (retcode) {
-    ret = STk_integer_value(retcode);
-    if (ret == LONG_MIN) STk_error("bad return code ~S", retcode);
-  } else {
-    retcode = MAKE_INT(0);
+    if (BOOLEANP(retcode)) {
+      ret = (retcode != STk_true);
+    } else {
+      ret = STk_integer_value(retcode);
+      if (ret == LONG_MIN) STk_error("bad return code ~S", retcode);
+    }
   }
 
-  STk_pre_exit(retcode);
-  exit(ret);
+  /* Raise a &exit-r7rs condition  with the numeric value of the exit code*/
+  cond = STk_make_C_cond(STk_exit_condition, 1, MAKE_INT(ret));
+  STk_raise(cond);
+
+  return STk_void; /* never reached */
+}
+
+/*
+<doc EXT emergency-exit
+ * (emergency-exit)
+ * (emergency-exit ret-code)
+ *
+ * Terminates the program without running any outstanding
+ * dynamic-wind ,(emph "after") procedures and communicates an exit
+ * value to the operating system in the same manner as |exit|.
+ * @l
+ * ,(bold "Note:") The ,(stklos) |emergency-exit| primitive accepts also an
+ * integer value as parameter (,(rseven) accepts only a boolean).
+doc>
+*/
+DEFINE_PRIMITIVE("emergency-exit", emergency_exit, subr01, (SCM retcode))
+{
+  long ret = 0;
+
+  if (retcode) {
+    if (BOOLEANP(retcode)) {
+      ret = (retcode != STk_true);
+    } else {
+      ret = STk_integer_value(retcode);
+      if (ret == LONG_MIN) STk_error("bad return code ~S", retcode);
+    }
+  }
+  _exit(ret);
 
   return STk_void; /* never reached */
 }
@@ -712,7 +1347,7 @@ DEFINE_PRIMITIVE("exit", exit, subr01, (SCM retcode))
  *
  * Returns a string identifying the kind of machine which is running the
  * program. The result string is of the form
- * |[os-name]-[os-version]-[processor-type]|.
+ * |[os-name]-[os-version]-[cpu-architecture]|.
 doc>
 */
 DEFINE_PRIMITIVE("machine-type", machine_type, subr0, (void))
@@ -738,13 +1373,13 @@ DEFINE_PRIMITIVE("machine-type", machine_type, subr0, (void))
 //EG:   long allocated, used, calls;
 //EG:
 //EG:   /* The result is a vector which contains
-//EG:    *	0 The total cpu used in ms
-//EG:    *	1 The number of cells currently in use.
+//EG:    *      0 The total cpu used in ms
+//EG:    *      1 The number of cells currently in use.
 //EG:    *    2 Total number of allocated cells
-//EG:    *	3 The number of cells used since the last call to get-internal-info
-//EG:    *	4 Number of gc calls
+//EG:    *      3 The number of cells used since the last call to get-internal-info
+//EG:    *      4 Number of gc calls
 //EG:    *    5 Total time used in the gc
-//EG:    *	6 A boolean indicating if Tk is initialized
+//EG:    *      6 A boolean indicating if Tk is initialized
 //EG:    */
 //EG:
 //EG:   STk_gc_count_cells(&allocated, &used, &calls);
@@ -771,7 +1406,7 @@ DEFINE_PRIMITIVE("machine-type", machine_type, subr0, (void))
 <doc EXT date
  * (date)
  *
- * Returns the current date in a string
+ * Returns the current date in a string.
 doc>
 */
 DEFINE_PRIMITIVE("date", date, subr0, (void))
@@ -796,15 +1431,22 @@ doc>
 DEFINE_PRIMITIVE("clock", clock, subr0, (void))
 {
   return STk_double2real((double) clock() /
-			 CLOCKS_PER_SEC * (double) TIME_DIV_CONST);
+                         CLOCKS_PER_SEC * (double) TIME_DIV_CONST);
 }
 
+
 /*
 <doc EXT current-seconds
  * (current-seconds)
  *
  * Returns the time since the Epoch (that is 00:00:00 UTC, January 1, 1970),
  * measured in seconds.
+ * @l
+ * @bold("Note"): This ,(stklos) function should not be confused with
+ * the ,(rseven)  primitive |current-second| which returns an inexact number
+ * and whose result is expressed using  the International Atomic Time
+ * instead of UTC.
+ *
 doc>
  */
 DEFINE_PRIMITIVE("current-seconds", current_seconds, subr0, (void))
@@ -813,7 +1455,37 @@ DEFINE_PRIMITIVE("current-seconds", current_seconds, subr0, (void))
 }
 
 /*
-<doc current-time
+<doc R7RS current-second
+ * (current-second)
+ *
+ * Returns an inexact number representing the current time on the
+ * International Atomic Time (TAI) scale.  The value 0.0 represents
+ * midnight on January 1, 1970 TAI (equivalent to ten seconds before
+ * midnight Universal Time) and the value 1.0 represents one TAI
+ * second later.
+doc>
+ */
+
+/* Offset: https://fr.wikipedia.org/wiki/Temps_atomique_international */
+#define TAI_OFFSET +37.0L
+
+DEFINE_PRIMITIVE("current-second", current_second, subr0, (void))
+{
+  /* R7RS states: Neither high accuracy nor high precision are
+   * required; in particular, returning Coordinated Universal Time plus
+   * a suitable constant might be the best an implementation can do.
+   */
+  struct timespec now;
+
+  clock_gettime(CLOCK_REALTIME, &now);
+  return STk_double2real(TAI_OFFSET +
+                         (double) now.tv_sec +
+                         1.0E-9 * (double) now.tv_nsec);
+}
+
+
+/*
+<doc EXT current-time
  * (current-time)
  *
  * Returns a time object corresponding to the current time.
@@ -821,15 +1493,16 @@ doc>
 */
 DEFINE_PRIMITIVE("current-time", current_time, subr0, (void))
 {
-  struct timeval now;
-  SCM argv[3];
+  struct timespec now;
+  SCM argv[4];
 
-  gettimeofday(&now, NULL);
+  clock_gettime(CLOCK_REALTIME, &now);
 
-  argv[2] = time_type;
-  argv[1] =  STk_long2integer(now.tv_sec);
-  argv[0] =  STk_long2integer(now.tv_usec);
-  return STk_make_struct(3, &argv[2]);
+  argv[3] = time_type;
+  argv[2] = STk_makekey("time-utc");
+  argv[1] = STk_long2integer(now.tv_sec);
+  argv[0] = STk_long2integer(now.tv_nsec);
+  return STk_make_struct(4, &argv[3]);
 }
 
 
@@ -891,7 +1564,7 @@ DEFINE_PRIMITIVE("%seconds->date", seconds2date, subr1, (SCM seconds))
   argv[2]  = MAKE_INT(t->tm_yday + 1);
   argv[1]  = MAKE_INT(t->tm_isdst);
 #ifdef DARWIN
-  argv[0]  = MAKE_INT(0);	/* Cannot figure how to find the timezone */
+  argv[0]  = MAKE_INT(0);       /* Cannot figure how to find the timezone */
 #else
   argv[0]  = STk_long2integer(timezone);
 #endif
@@ -922,7 +1595,7 @@ DEFINE_PRIMITIVE("date->seconds", date2seconds, subr1, (SCM date))
   t.tm_mday  = STk_integer_value(*p++);
   t.tm_mon   = STk_integer_value(*p++) - 1;
   t.tm_year  = STk_integer_value(*p++) - 1900;
-  t.tm_isdst = -1;			/* to ignore DST */
+  t.tm_isdst = -1;                      /* to ignore DST */
 
   n = mktime(&t);
   if (n == (time_t)(-1)) STk_error("cannot convert date to seconds (~S)", date);
@@ -963,7 +1636,7 @@ DEFINE_PRIMITIVE("%seconds->string", date2string, subr2, (SCM fmt, SCM seconds))
  * Returns the name of the underlying Operating System which is running
  * the program.
  * The value returned by |runnin-os| is a symbol. For now, this procedure
- * returns either |unix|, |windows|, or |cygwin-windows|.
+ * returns either |unix|, |android|, |windows|, or |cygwin-windows|.
 doc>
 */
 DEFINE_PRIMITIVE("running-os", running_os, subr0, (void))
@@ -975,8 +1648,12 @@ DEFINE_PRIMITIVE("running-os", running_os, subr0, (void))
   return STk_intern("windows");
 # endif
 #else
+#  ifdef __ANDROID__
+  return STk_intern("android");
+#  else
   return STk_intern("unix");
 #endif
+#endif
 }
 
 
@@ -1018,12 +1695,12 @@ DEFINE_PRIMITIVE("getenv", getenv, subr01, (SCM str))
 {
   char *tmp;
 
-  if (str) {		/* One parameter: find the value of the given variable */
+  if (str) {            /* One parameter: find the value of the given variable */
     if (!STRINGP(str)) error_bad_string(str);
 
     tmp = getenv(STRING_CHARS(str));
     return tmp ? STk_Cstring2string(tmp) : STk_false;
-  } else {		/* No parameter: give the complete environment */
+  } else {              /* No parameter: give the complete environment */
     extern char **environ;
     return build_posix_environment(environ);
   }
@@ -1040,14 +1717,17 @@ doc>
 DEFINE_PRIMITIVE("setenv!", setenv, subr2, (SCM var, SCM value))
 {
   char *s;
-
-  if (!STRINGP(var)) 		      error_bad_string(var);
+  unsigned long len;
+  
+  if (!STRINGP(var))                  error_bad_string(var);
   if (strchr(STRING_CHARS(var), '=')) STk_error("variable ~S contains a '='", var);
-  if (!STRINGP(value)) 		      STk_error("value ~S is not a string", value);
+  if (!STRINGP(value))                STk_error("value ~S is not a string", value);
 
-  s = STk_must_malloc(strlen(STRING_CHARS(var))   +
-		      strlen(STRING_CHARS(value)) + 2); /* 2 because of '=' & \0 */
-  sprintf(s, "%s=%s", STRING_CHARS(var), STRING_CHARS(value));
+  len = strlen(STRING_CHARS(var))   +
+        strlen(STRING_CHARS(value)) + 2; /* 2 because of '=' & \0 */
+  
+  s = STk_must_malloc_atomic(len);
+  snprintf(s, len, "%s=%s", STRING_CHARS(var), STRING_CHARS(value));
   putenv(s);
   return STk_void;
 }
@@ -1101,8 +1781,6 @@ DEFINE_PRIMITIVE("pause", pause, subr0, (void))
 
 
 
-
-
 /*
  * Undocumented primitives
  *
@@ -1143,36 +1821,72 @@ DEFINE_PRIMITIVE("%big-endian?", big_endianp, subr0, (void))
 }
 
 
+DEFINE_PRIMITIVE("%get-locale", get_locale, subr0, (void))
+{
+  char *str = setlocale(LC_ALL, NULL);
+
+  return str? STk_Cstring2string(str) : STk_false;
+}
+
+
+DEFINE_PRIMITIVE("%uname", uname, subr0, (void))
+{
+  struct utsname name;
+  SCM z;
+
+  if (uname(&name) != 0) {
+    error_posix(NULL, NULL);
+  }
+  z = STk_makevect(5, STk_void);
+  VECTOR_DATA(z)[0] = STk_Cstring2string(name.sysname);
+  VECTOR_DATA(z)[1] = STk_Cstring2string(name.nodename);
+  VECTOR_DATA(z)[2] = STk_Cstring2string(name.release);
+  VECTOR_DATA(z)[3] = STk_Cstring2string(name.version);
+  VECTOR_DATA(z)[4] = STk_Cstring2string(name.machine);
+
+  return z;
+}
+
+
 int STk_init_system(void)
 {
   SCM current_module = STk_STklos_module;
 
   /* Create the system-date structure-type */
   date_type =  STk_make_struct_type(STk_intern("%date"),
-				    STk_false,
-				    LIST10(STk_intern("second"),
-					   STk_intern("minute"),
-					   STk_intern("hour"),
-					   STk_intern("day"),
-					   STk_intern("month"),
-					   STk_intern("year"),
-					   STk_intern("week-day"),
-					   STk_intern("year-day"),
-					   STk_intern("dst"),
-					   STk_intern("tz")));
+                                    STk_false,
+                                    LIST10(STk_intern("second"),
+                                           STk_intern("minute"),
+                                           STk_intern("hour"),
+                                           STk_intern("day"),
+                                           STk_intern("month"),
+                                           STk_intern("year"),
+                                           STk_intern("week-day"),
+                                           STk_intern("year-day"),
+                                           STk_intern("dst"),
+                                           STk_intern("tz")));
   STk_define_variable(STk_intern("%date"), date_type, current_module);
 
   /* Create the time structure-type */
   time_type =  STk_make_struct_type(STk_intern("%time"),
-				    STk_false,
-				    LIST2(STk_intern("second"),
-					  STk_intern("microsecond")));
+                                    STk_false,
+                                    LIST3(STk_intern("type"),
+                                          STk_intern("second"),
+                                          STk_intern("nanosecond")));
   STk_define_variable(STk_intern("%time"), time_type, current_module);
 
+
+  /* Add parameter for generating temporary files */
+  STk_make_C_parameter("temp-file-prefix",
+                       init_temp_file_prefix(),
+                       temp_file_prefix_conv,
+                       current_module);
+
   /* Declare primitives */
   ADD_PRIMITIVE(clock);
   ADD_PRIMITIVE(date);
   ADD_PRIMITIVE(current_seconds);
+  ADD_PRIMITIVE(current_second);
   ADD_PRIMITIVE(current_time);
   ADD_PRIMITIVE(sleep);
   ADD_PRIMITIVE(seconds2date);
@@ -1191,7 +1905,7 @@ int STk_init_system(void)
   ADD_PRIMITIVE(getcwd);
   ADD_PRIMITIVE(chdir);
   ADD_PRIMITIVE(make_directory);
-  ADD_PRIMITIVE(remove_directory);
+  ADD_PRIMITIVE(delete_directory);
   ADD_PRIMITIVE(directory_files);
   ADD_PRIMITIVE(getpid);
   ADD_PRIMITIVE(system);
@@ -1208,11 +1922,14 @@ int STk_init_system(void)
   ADD_PRIMITIVE(canonical_path);
 
   ADD_PRIMITIVE(remove_file);
+  ADD_PRIMITIVE(delete_file);
   ADD_PRIMITIVE(rename_file);
   ADD_PRIMITIVE(copy_file);
-  ADD_PRIMITIVE(tmp_file);
+  ADD_PRIMITIVE(create_tmp_file);
+  ADD_PRIMITIVE(create_tmp_dir);
   ADD_PRIMITIVE(pre_exit);
   ADD_PRIMITIVE(exit);
+  ADD_PRIMITIVE(emergency_exit);
   ADD_PRIMITIVE(at_exit);
   ADD_PRIMITIVE(machine_type);
 
@@ -1221,5 +1938,7 @@ int STk_init_system(void)
 
   ADD_PRIMITIVE(pause);
   ADD_PRIMITIVE(big_endianp);
+  ADD_PRIMITIVE(get_locale);
+  ADD_PRIMITIVE(uname);
   return TRUE;
 }
