@@ -1,7 +1,7 @@
 /*
- * u v e c t o r . c			-- Uniform Vectors Implementation
+ * u v e c t o r . c                    -- Uniform Vectors Implementation
  *
- * Copyright © 2001-2011 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright © 2001-2021 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,46 +21,12 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 15-Apr-2001 10:13 (eg)
- * Last file update: 19-Aug-2011 11:27 (eg)
+ * Last file update: 10-Apr-2021 18:47 (eg)
  */
 
 #include "stklos.h"
 
-#define UVECT_S8	0
-#define UVECT_U8	1
-#define UVECT_S16	2
-#define UVECT_U16	3
-#define UVECT_S32	4
-#define UVECT_U32	5
-#define UVECT_S64	6
-#define UVECT_U64	7
-
-#define UVECT_F32	8
-#define UVECT_F64	9
-
-/*
- * 64 bits values are always represeneted with bignums even on 64 bits machines
- * Here are the intersting maxima for 64 bits.
- */
-#define S64_MIN "-9223372036854775808"
-#define S64_MAX  "9223372036854775807"
-#define U64_MAX	"18446744073709551615"
-
-
-struct uvector_obj {
-  stk_header header;
-  int vect_type;
-  int size;
-  char data[1];
-};
-
 int STk_uvectors_allowed = 0;
-
-#define UVECTOR_TYPE(p) (((struct uvector_obj *) (p))->vect_type)
-#define UVECTOR_SIZE(p)	(((struct uvector_obj *) (p))->size)
-#define UVECTOR_DATA(p) (((struct uvector_obj *) (p))->data)
-#define UVECTORP(p)	(BOXED_TYPE_EQ((p), tc_uvector))
-
 static SCM u64_max, s64_min, s64_max;
 
 
@@ -69,6 +35,24 @@ static SCM u64_max, s64_min, s64_max;
  *
  */
 
+static char* type_vector(int tip)
+{
+  switch (tip) {
+    case UVECT_S8:  return "s8";
+    case UVECT_U8:  return "u8";
+    case UVECT_S16: return "s16";
+    case UVECT_U16: return "u16";
+    case UVECT_S32: return "s32";
+    case UVECT_U32: return "u32";
+    case UVECT_S64: return "s64";
+    case UVECT_U64: return "u64";
+    case UVECT_F32: return "f32";
+    case UVECT_F64: return "f64";
+    default:        return ""; /* never reached */
+  }
+}
+
+
 static void error_change_const_vector(SCM v)
 {
   STk_error("changing the constant vector ~s is not allowed", v);
@@ -79,6 +63,11 @@ static void error_bad_vector(SCM v)
   STk_error("bad vector ~s", v);
 }
 
+static void error_bad_uvector(SCM v, int tip)
+{
+  STk_error("bad #%s vector ~s", type_vector(tip), v);
+}
+
 static void error_bad_index(SCM index)
 {
   STk_error("index ~S is invalid or out of bounds", index);
@@ -100,23 +89,6 @@ static void error_bad_list(SCM l)
 }
 
 
-static char* type_vector(SCM vect)
-{
-  switch (UVECTOR_TYPE(vect)) {
-    case UVECT_S8:  return "s8";
-    case UVECT_U8:  return "u8";
-    case UVECT_S16: return "s16";
-    case UVECT_U16: return "u16";
-    case UVECT_S32: return "s32";
-    case UVECT_U32: return "u32";
-    case UVECT_S64: return "s64";
-    case UVECT_U64: return "u64";
-    case UVECT_F32: return "f32";
-    case UVECT_F64: return "f64";
-    default:        return ""; /* never reached */
-  }
-}
-
 static int vector_element_size(int type)
 {
   /* compute len of one element depending of type.  We assume here
@@ -132,13 +104,67 @@ static int vector_element_size(int type)
     case UVECT_S16: case UVECT_U16: return 2;
     case UVECT_S32: case UVECT_U32: return 4;
     case UVECT_S64: case UVECT_U64: return sizeof(SCM);
-    case UVECT_F32: 		    return 4;
-    case UVECT_F64: 		    return 8;
+    case UVECT_F32:                 return 4;
+    case UVECT_F64:                 return 8;
   }
   return 0; /* never reached */
 }
 
 
+static SCM control_index(int argc, SCM *argv, long *pstart, long *pend, SCM *pfill)
+{
+  SCM v = STk_void; /* value chosen to avoid a warning from the gcc static analyzer */
+  long len, start=0, end=-1;
+
+  /* Controling number of arguments */
+  if (!pfill) {
+    /* We do not have a fill parameter => vect at 0, start at -1 and end at -2 */
+    switch (argc) {
+      case 3: end   = STk_integer_value(argv[-2]);  /* FALLTHROUGH */
+      case 2: start = STk_integer_value(argv[-1]);  /* FALLTHROUGH */
+      case 1: v = argv[0]; break;
+      default: goto bad_number_of_args;
+    }
+  } else {
+    /* We have a fill param. => vect at 0, fill at -1, start at -2 and end at -3 */
+    switch (argc) {
+      case 4: end   = STk_integer_value(argv[-3]);  /* FALLTHROUGH */
+      case 3: start = STk_integer_value(argv[-2]);  /* FALLTHROUGH */
+      case 2: if (pfill) *pfill  = argv[-1];
+              v = argv[0];
+              break;
+      default:
+      bad_number_of_args:
+        STk_error("incorrect number of arguments (%d)", argc);
+    }
+  }
+
+  /* Controlling s */
+  if (!UVECTORP(v)) error_bad_uvector(v, UVECT_U8);
+  len = UVECTOR_SIZE(v);
+
+  /* Controlling start index */
+  if (start == LONG_MIN || start < 0 || start > len)
+    /* argc cannot be 1 (start would be 0) */
+    STk_error("bad starting index ~S", argv[pfill ? -2: -1]);
+
+  /* Controlling end index */
+  if (end == -1)
+    end = len;
+  else
+    if (end == LONG_MIN  || end < 0 || end > len)
+      /* We have an end index ==> argc = 3 */
+      STk_error("bad ending index ~S", argv[pfill? -3: -2]);
+
+  if (start > end)
+    STk_error("low index is greater than high index");
+
+  /* everything is correct, return values */
+  *pstart = start;
+  *pend   = end;
+  return v;
+}
+
 
 /* Return the type of an uniform vector given its tag */
 int STk_uniform_vector_tag(char *s)
@@ -170,89 +196,90 @@ int STk_uvector_equal(SCM u1, SCM u2)
  * Basic accessors to an uniform vector
  *
  */
-static void uvector_set(int type, SCM v, long i, SCM value)
+static void uvector_set(int _UNUSED(type), SCM v, long i, SCM value)
 {
-  int vali, overflow;
+  long vali;
+  int overflow;
 
   /* First see if the value is correct for this type of vector */
   switch (UVECTOR_TYPE(v)) {
     case UVECT_S8:
-      vali = STk_integer_value(value); // FIXME : nocheck
+      vali = STk_integer_value(value);
       if (-128 <= vali && vali < +128) {
-	((char *) UVECTOR_DATA(v))[i] = (char) vali;
-	return;
+        ((char *) UVECTOR_DATA(v))[i] = (char) vali;
+        return;
       }
       break;
     case UVECT_U8:
-      vali = STk_integer_value(value); // FIXME : nocheck
+      vali = STk_integer_value(value);
       if (0 <= vali && vali < +256) {
-	((unsigned char *) UVECTOR_DATA(v))[i] = (unsigned char) vali;
-	return;
+        ((unsigned char *) UVECTOR_DATA(v))[i] = (unsigned char) vali;
+        return;
       }
       break;
     case UVECT_S16:
-      vali = STk_integer_value(value); // FIXME : nocheck
+      vali = STk_integer_value(value);
       if (-32768 <= vali && vali < +32768) {
-	((short *) UVECTOR_DATA(v))[i] = (short) vali;
-	return;
+        ((short *) UVECTOR_DATA(v))[i] = (short) vali;
+        return;
       }
       break;
     case UVECT_U16:
-      vali = STk_integer_value(value); // FIXME : nocheck
+      vali = STk_integer_value(value);
       if (0 <= vali && vali < 65536) {
-	((unsigned short *) UVECTOR_DATA(v))[i] = (unsigned short) vali;
-	return;
+        ((unsigned short *) UVECTOR_DATA(v))[i] = (unsigned short) vali;
+        return;
       }
       break;
 
     case UVECT_S32:
       vali = STk_integer2int32(value, &overflow);
       if (!overflow) {
-	((int *) UVECTOR_DATA(v))[i] = (int) vali;
-	return;
+        ((int *) UVECTOR_DATA(v))[i] = (int) vali;
+        return;
       }
       break;
     case UVECT_U32:
       vali = STk_integer2uint32(value, &overflow);
       if (!overflow) {
-	((unsigned int *) UVECTOR_DATA(v))[i] = (unsigned int) vali;
-	return;
+        ((unsigned int *) UVECTOR_DATA(v))[i] = (unsigned int) vali;
+        return;
       }
       break;
     case UVECT_S64:
       if (INTP(value) || BIGNUMP(value))
-	if (STk_numle2(MAKE_INT(0), value) && STk_numle2(value, u64_max)) {
-	  ((SCM *) UVECTOR_DATA(v))[i] = value;
-	  return;
-	}
+        if (STk_numle2(s64_min, value) && STk_numle2(value, s64_max)) {
+          ((SCM *) UVECTOR_DATA(v))[i] = value;
+          return;
+        }
       break;
     case UVECT_U64:
       if (INTP(value) || BIGNUMP(value))
-	if (STk_numle2(s64_min, value) && STk_numle2(value, s64_max)) {
-	  ((SCM *) UVECTOR_DATA(v))[i] = value;
-	  return;
-	}
+        if (STk_numle2(MAKE_INT(0), value) && STk_numle2(value, u64_max)) {
+          ((SCM *) UVECTOR_DATA(v))[i] = value;
+          return;
+        }
       break;
     case UVECT_F32:
       if (REALP(value)) {
-	((float *) UVECTOR_DATA(v))[i] = (float) REAL_VAL(value);
-	return;
+        ((float *) UVECTOR_DATA(v))[i] = (float) REAL_VAL(value);
+        return;
       }
       break;
     case UVECT_F64:
       if (REALP(value)) {
-	((double *) UVECTOR_DATA(v))[i] = (double) REAL_VAL(value);
-	return;
+        ((double *) UVECTOR_DATA(v))[i] = (double) REAL_VAL(value);
+        return;
       }
       break;
   }
 
   /* If we arrive here we are sure that we have a value which is out of bounds */
   STk_error("value ~S is out of bounds or incorrect for a %svector",
-	    value, type_vector(v));
+            value, type_vector(UVECTOR_TYPE(v)));
 }
 
-static SCM uvector_ref(int type, SCM v, long i)
+static SCM uvector_ref(int _UNUSED(type), SCM v, long i)
 {
   switch (UVECTOR_TYPE(v)) {
     case UVECT_S8: return MAKE_INT(((char *) UVECTOR_DATA(v))[i]);
@@ -292,12 +319,12 @@ static SCM makeuvect(int type, int len, SCM init)
    * double IEEE-754 reals
    */
   switch (type) {
-    case UVECT_S8:  case UVECT_U8:  size = 1; 		break;
-    case UVECT_S16: case UVECT_U16: size = 2; 		break;
-    case UVECT_S32: case UVECT_U32: size = 4; 		break;
+    case UVECT_S8:  case UVECT_U8:  size = 1;           break;
+    case UVECT_S16: case UVECT_U16: size = 2;           break;
+    case UVECT_S32: case UVECT_U32: size = 4;           break;
     case UVECT_S64: case UVECT_U64: size = sizeof(SCM); break;
-    case UVECT_F32: 		    size = 4; 		break;
-    case UVECT_F64: 		    size = 8; 		break;
+    case UVECT_F32:                 size = 4;           break;
+    case UVECT_F64:                 size = 8;           break;
   }
   NEWCELL_WITH_LEN(z, uvector, sizeof(struct vector_obj) + size*len - 1);
   UVECTOR_TYPE(z) = type;
@@ -337,7 +364,7 @@ DEFINE_PRIMITIVE("%make-uvector", make_uvector, subr3,(SCM type, SCM len, SCM in
   long l   = STk_integer_value(len);
   long tip = STk_integer_value(type);
 
-  if (l < 0) 				 error_bad_length(len);
+  if (l < 0)                             error_bad_length(len);
   if (tip < UVECT_S8 || tip > UVECT_F64) error_bad_uniform_type(type);
 
   return makeuvect(tip, l, init);
@@ -362,7 +389,7 @@ DEFINE_PRIMITIVE("%uvector-length", uvector_length, subr2, (SCM type, SCM v))
   long tip = STk_integer_value(type);
 
   if (tip < UVECT_S8 || tip > UVECT_F64)        error_bad_uniform_type(type);
-  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_vector(v);
+  if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_uvector(v, tip);
 
   return MAKE_INT(UVECTOR_SIZE(v));
 }
@@ -380,15 +407,15 @@ DEFINE_PRIMITIVE("%uvector-ref", uvector_ref, subr3, (SCM type, SCM v, SCM index
 }
 
 DEFINE_PRIMITIVE("%uvector-set!", uvector_set, subr4,
-		 (SCM type, SCM v, SCM index, SCM value))
+                 (SCM type, SCM v, SCM index, SCM value))
 {
   long i   = STk_integer_value(index);
   long tip = STk_integer_value(type);
 
   if (!UVECTORP(v) || (UVECTOR_TYPE(v) != tip)) error_bad_vector(v);
-  if (tip < UVECT_S8 || tip > UVECT_F64) 	error_bad_uniform_type(type);
-  if (BOXED_INFO(v) & VECTOR_CONST) 		error_change_const_vector(v);
-  if (i < 0 || i >= UVECTOR_SIZE(v)) 		error_bad_index(index);
+  if (tip < UVECT_S8 || tip > UVECT_F64)        error_bad_uniform_type(type);
+  if (BOXED_INFO(v) & VECTOR_CONST)             error_change_const_vector(v);
+  if (i < 0 || i >= UVECTOR_SIZE(v))            error_bad_index(index);
 
   uvector_set(tip, v, i, value);
   return STk_void;
@@ -424,7 +451,6 @@ DEFINE_PRIMITIVE("%list->uvector", list_uvector, subr2, (SCM type, SCM l))
 
 DEFINE_PRIMITIVE("%allow-uvectors", allow_uvectors, subr0, (void))
 {
-  ADD_PRIMITIVE(uvector);
   ADD_PRIMITIVE(uvector_list);
   ADD_PRIMITIVE(list_uvector);
 
@@ -452,7 +478,7 @@ static void print_uvector(SCM vect, SCM port, int mode)
   int n = UVECTOR_SIZE(vect);
   int t = UVECTOR_TYPE(vect);
 
-  STk_fprintf(port, "#%s(", type_vector(vect));
+  STk_fprintf(port, "#%s(", type_vector(UVECTOR_TYPE(vect)));
   for (i = 0; i < n; i++) {
     STk_print(uvector_ref(t, vect, i), port, mode);
     if (i < n - 1) STk_putc(' ', port);
@@ -462,13 +488,145 @@ static void print_uvector(SCM vect, SCM port, int mode)
 
 
 static struct extended_type_descr xtype_uvector = {
-  "uvector",
-  print_uvector
+  .name  = "uvector",
+  .print = print_uvector
 };
 
 /*==========================================================================*/
+/*                                                                          */
+/*                              B Y T E V E C T O R S                       */
+/*                                                                          */
+/*==========================================================================*/
+
+SCM STk_make_C_bytevector(int len)
+{
+  return makeuvect(UVECT_U8, len, NULL);
+}
+
+SCM STk_make_bytevector_from_C_string(char *str, long len)
+{
+  SCM z  = makeuvect(UVECT_U8, len, (SCM) NULL);
+  memcpy(UVECTOR_DATA(z),str, len);
+  return z;
+}
 
 
+/*
+<doc R7RS bytevector-copy
+ * (bytevector-copy bytevector)
+ * (bytevector-copy bytevector start)
+ * (bytevector-copy bytevector start end)
+ *
+ * Returns a newly allocated bytevector containing the bytes in |bytevector|
+ * between |start| and |end|.
+ * @lisp
+ * (define a #u8(1 2 3 4 5))
+ * (bytevector-copy a 2 4))       =>  #u8(3 4)
+ * @end lisp
+doc>
+*/
+DEFINE_PRIMITIVE("bytevector-copy", bytevector_copy, vsubr, (int argc, SCM *argv))
+{
+  long start, end, n;
+  SCM z, bvect;
+
+  bvect = control_index(argc, argv, &start, &end, NULL);
+  n = end-start;
+  z = makeuvect(UVECT_U8, n, (SCM) NULL);
+  memcpy(UVECTOR_DATA(z),
+         (unsigned char *)UVECTOR_DATA(bvect)+start,
+         n * sizeof(unsigned char));
+  return z;
+}
+
+/*
+<doc  bytevector-append
+ * (bytevector-append bytevector ...)
+ *
+ * Returns a newly allocated bytevector whose elements are
+ * the concatenation of the elements in the given bytevectors.
+ * @lisp
+ * (bytevector-append #u8(0 1 2) #u8(3 4 5))
+ *                     =>  #u8(0 1 2 3 4 5)
+ * @end lisp
+doc>
+*/
+DEFINE_PRIMITIVE("bytevector-append", bytevector_append, vsubr,(int argc, SCM *argv))
+{
+  int i, len = 0, start = 0;
+  SCM z;
+
+  /* compute length of final result */
+  for (i = 0; i < argc; i++) {
+    if (!UVECTORP(argv[-i]) || UVECTOR_TYPE(argv[-i]) != UVECT_U8)
+      error_bad_uvector(argv[-i], UVECT_U8);
+    len += UVECTOR_SIZE(argv[-i]);
+  }
+
+  /* copy bytevectors */
+  z = makeuvect(UVECT_U8, len, (SCM) NULL);
+
+  for (i = 0; i < argc; i++) {
+    int sz = UVECTOR_SIZE(argv[-i]);
+    memcpy(UVECTOR_DATA(z+start),
+           (unsigned char *) UVECTOR_DATA(argv[-i]),
+           sz * sizeof(unsigned char));
+    start += sz * sizeof(unsigned char);
+  }
+  return z;
+}
+
+/*
+<doc R7RS utf8->string string->utf8
+ * (utf8->string bytevector)
+ * (utf8->string bytevector start)
+ * (utf8->string bytevector start end)
+ * (string->utf8 string)
+ * (string->utf8 string start)
+ * (string->utf8 string start end)
+ *
+ * These procedures translate between strings and bytevectors
+ * that encode those strings using the UTF-8 encoding.
+ * The |utf8->string| procedure decodes the bytes of
+ * a bytevector between |start| and |end| and returns the
+ * corresponding string; the |string->utf8| procedure encodes the
+ * characters of a string between |start| and |end| and returns
+ * the corresponding bytevector.
+ *
+ * It is an error for |bytevector| to contain invalid UTF-8 byte
+ * sequences.
+ * @lisp
+ * (utf8->string #u8(#x41))   => "A"
+ * (string->utf8 "λ")         => #u8((#xce #xbb)
+ * @end lisp
+doc>
+*/
+DEFINE_PRIMITIVE("utf8->string", utf82string, vsubr, (int argc, SCM *argv))
+{
+  long start, end, len;
+  SCM v;
+  unsigned char *start_addr, *end_addr;
+
+  v          = control_index(argc, argv, &start, &end, NULL);
+  start_addr = (unsigned char*) UVECTOR_DATA(v) + start;
+  end_addr   = (unsigned char*) UVECTOR_DATA(v) + end;
+  len        = end_addr - start_addr;
+
+  /* Verify that the sub-vector denotes a correct string */
+  if (STk_utf8_verify_sequence((char *) start_addr, len)) {
+    SCM z = STk_makestring(len, NULL);
+    memcpy(STRING_CHARS(z), start_addr, end_addr - start_addr);
+    return z;
+  }
+  else
+    STk_error("bad UTF8 sequence between %d and %d in ~S", start, end, v);
+
+  return STk_void;  /* for the compiler */
+}
+
+
+
+/* ====================================================================== */
 int STk_init_uniform_vector(void)
 {
   DEFINE_XTYPE(uvector, &xtype_uvector);
@@ -477,11 +635,17 @@ int STk_init_uniform_vector(void)
    * SRFI-4 is required
    */
   ADD_PRIMITIVE(make_uvector);
+  ADD_PRIMITIVE(uvector);
   ADD_PRIMITIVE(uvectorp);
   ADD_PRIMITIVE(uvector_length);
   ADD_PRIMITIVE(uvector_ref);
   ADD_PRIMITIVE(uvector_set);
 
+  /* R7RS specific bytevectors primitives */
+  ADD_PRIMITIVE(bytevector_copy);
+  ADD_PRIMITIVE(bytevector_append);
+  ADD_PRIMITIVE(utf82string);
+
   /* A pseudo primitive to launch the definition of  all the function of SRFI-4 */
   ADD_PRIMITIVE(allow_uvectors);
 
